-----------------------------------------------------------------------------------------------
-- Client Lua Script for RazorPlates
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------

require "Window"
require "ChallengesLib"
require "Unit"
require "GameLib"
require "Apollo"
require "PathMission"
require "Quest"
require "Episode"
require "math"
require "string"
require "DialogSys"
require "PublicEvent"
require "PublicEventObjective"
require "CommunicatorLib"
require "GroupLib"
require "PlayerPathLib"
require "GuildLib"
require "GuildTypeLib"

local strFormat = string.format
local pairs = pairs
local RazorPlates = {}
--local razHPTextOptions = {"hide", "default", "2.7k - 52%", "52%", "2.7k (3.1k)", "2.7k"}
local razHPTextOptions = {"hide", "current", "cur/max", "cur%", "deficit", "cur/max-%", "cur - %", "cur/max (shield)", "cur (shield)"}
local razResourceBarTextOptions = { "hide", "current", "cur/max", "cur%", "deficit","cur/max-%"}
local razHPTextAlignmentOptions = {"left", "center", "right"}

local ProfilesSaved = {}

local razFontSizes = {}
local razFonts = {"Courier","CRB_Interface10","CRB_Interface11","CRB_Interface12","CRB_Interface14","CRB_Interface16","CRB_Pixel","Default", "DefaultButton", "Nameplates", "Subtitle"}

local razGroupSettings = {
								[true] =  {
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGroupMemberShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGroupMemberShowGuild" },
															["razNameColor"] = { default="00ffff", settingsVal= nil, nControlType=2, strControlName="InCombatGroupMemberNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGroupMemberShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGroupMemberShowBars" },
															["razUseClassColors"] = { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGroupMemberUseClassColors" },
															["razHPBarColor"] 	= { default="00ffff", settingsVal= nil, nControlType=2, strControlName="InCombatGroupMemberHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGroupMemberShowHPText" },
															["razShowResourceBar"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGroupMemberShowResourceBar" },
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGroupMemberShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatGroupMemberTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatGroupMemberScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="InCombatGroupMemberOffset" },
															
														},
								[false] = 	{
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGroupMemberShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGroupMemberShowGuild" },
															["razNameColor"] = { default="00ffff", settingsVal= nil, nControlType=2, strControlName="OutOfCombatGroupMemberNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGroupMemberShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGroupMemberShowBars" },
															["razUseClassColors"] = { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGroupMemberUseClassColors" },
															["razHPBarColor"] 	= { default="00ffff", settingsVal= nil, nControlType=2, strControlName="OutOfCombatGroupMemberHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGroupMemberShowHPText" },
															["razShowResourceBar"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGroupMemberShowResourceBar" },
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGroupMemberShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatGroupMemberTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatGroupMemberScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="OutOfCombatGroupMemberOffset" },
															
														},
							}
local razGuildSettings = {
								[true] = {
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGuildMemberShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGuildMemberShowGuild" },
															["razNameColor"] = { default="00ffff", settingsVal= nil, nControlType=2, strControlName="InCombatGuildMemberNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGuildMemberShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGuildMemberShowBars" },
															["razUseClassColors"] = { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGuildMemberUseClassColors" },
															["razHPBarColor"] 	= { default="00ffff", settingsVal= nil, nControlType=2, strControlName="InCombatGuildMemberHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGuildMemberShowHPText" },
															["razShowResourceBar"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGuildMemberShowResourceBar" },
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatGuildMemberShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatGuildMemberTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatGuildMemberScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="InCombatGuildMemberOffset" },
															
														},
								[false] = 	{
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGuildMemberShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGuildMemberShowGuild" },
															["razNameColor"] = { default="00ffff", settingsVal= nil, nControlType=2, strControlName="OutOfCombatGuildMemberNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGuildMemberShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGuildMemberShowBars" },
															["razUseClassColors"] = { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGuildMemberUseClassColors" },
															["razHPBarColor"] 	= { default="00ffff", settingsVal= nil, nControlType=2, strControlName="OutOfCombatGuildMemberHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGuildMemberShowHPText" },
															["razShowResourceBar"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGuildMemberShowResourceBar" },
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatGuildMemberShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatGuildMemberTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatGuildMemberScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="OutOfCombatGuildMemberOffset" },
															
														},
							}

local razTargetSettings = {
								[true] =				{
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatTargetShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatTargetShowGuild" },
															--["razNameColor"] = { default="ff0000", settingsVal= nil, nControlType=2, strControlName="InCombatTargetNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatTargetShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatTargetShowBars" },
														--	["razUseClassColors"] = { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatTargetUseClassColors" },
														--	["razHPBarColor"] 	= { default="ff0000", settingsVal= nil, nControlType=2, strControlName="InCombatTargetHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatTargetShowHPText" },
															["razShowResourceBar"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatTargetShowResourceBar" },
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatTargetShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatTargetTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatTargetScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="InCombatTargetOffset" },
														},
								[false] = 				{
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatTargetShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatTargetShowGuild" },
															--["razNameColor"] = { default="ff0000", settingsVal= nil, nControlType=2, strControlName="OutOfCombatTargetNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatTargetShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatTargetShowBars" },
															--["razUseClassColors"] = { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatTargetUseClassColors" },
														--	["razHPBarColor"] 	= { default="ff0000", settingsVal= nil, nControlType=2, strControlName="OutOfCombatTargetHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatTargetShowHPText" },
															["razShowResourceBar"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatTargetShowResourceBar" },
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatTargetShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatTargetTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatTargetScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="OutOfCombatTargetOffset" },
														},
			}

local razPerUnitTypeSettings =
{
	-- in Combat
	[true] = {
			["MyNameplate"] = {
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatMyNameplateShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatMyNameplateShowGuild" },
															["razNameColor"] = { default="ff0000", settingsVal= nil, nControlType=2, strControlName="InCombatMyNameplateNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatMyNameplateShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatMyNameplateShowBars" },
															["razUseClassColors"] = { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatMyNameplateUseClassColors" },
															["razHPBarColor"] 	= { default="ff0000", settingsVal= nil, nControlType=2, strControlName="InCombatMyNameplateHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatMyNameplateShowHPText" },
															["razShowResourceBar"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatMyNameplateShowResourceBar" },
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatMyNameplateShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatMyNameplateTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatMyNameplateScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="InCombatMyNameplateOffset" },

			},
			["TargetNameplate"] = razTargetSettings[true],
			["GroupMember"] =	razGroupSettings[true],
			["GuildMember"] =	razGuildSettings[true],
			["Player"]	= {
				[Unit.CodeEnumDisposition.Hostile] =	{
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostilePlayerShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostilePlayerShowGuild" },
															["razNameColor"] = { default="ff0000", settingsVal= nil, nControlType=2, strControlName="InCombatHostilePlayerNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostilePlayerShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostilePlayerShowBars" },
															["razUseClassColors"] = { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostilePlayerUseClassColors" },
															["razHPBarColor"] 	= { default="ff0000", settingsVal= nil, nControlType=2, strControlName="InCombatHostilePlayerHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostilePlayerShowHPText" },
															["razShowResourceBar"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostilePlayerShowResourceBar" },
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostilePlayerShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatHostilePlayerTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatHostilePlayerScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="InCombatHostilePlayerOffset" },
															
														},
				[Unit.CodeEnumDisposition.Friendly] = 	{
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyPlayerShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyPlayerShowGuild" },
															["razNameColor"] = { default="00ff00", settingsVal= nil, nControlType=2, strControlName="InCombatFriendlyPlayerNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyPlayerShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyPlayerShowBars" },
															["razUseClassColors"] = { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyPlayerUseClassColors" },
															["razHPBarColor"] 	= { default="00ff00", settingsVal= nil, nControlType=2, strControlName="InCombatFriendlyPlayerHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyPlayerShowHPText" },
															["razShowResourceBar"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyPlayerShowResourceBar" },
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyPlayerShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatFriendlyPlayerTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatFriendlyPlayerScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="InCombatFriendlyPlayerOffset" },
															
														},
									
							},
			["NonPlayer"] 	= {
				[Unit.CodeEnumDisposition.Hostile] = {
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostileNPCShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostileNPCShowGuild" },
															["razNameColor"] = { default="ff0000", settingsVal= nil, nControlType=2, strControlName="InCombatHostileNPCNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostileNPCShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostileNPCShowBars" },
															["razUseClassColors"] = { default=false, settingsVal=nil, nControlType=1, strControlName="InCombatHostileNPCUseClassColors" },
															["razHPBarColor"] 	= { default="ff0000", settingsVal= nil, nControlType=2, strControlName="InCombatHostileNPCHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostileNPCShowHPText" },
															["razShowResourceBar"]	= { default=false, settingsVal=nil, nControlType=1, strControlName=""},
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatHostileNPCShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatHostileNPCTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatHostileNPCScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="InCombatHostileNPCOffset" },
															
														},
				[Unit.CodeEnumDisposition.Neutral] = 	{
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatNeutralNPCShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatNeutralNPCShowGuild" },
															["razNameColor"] ={ default="ffff00", settingsVal= nil, nControlType=2, strControlName="InCombatNeutralNPCNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatNeutralNPCShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatNeutralNPCShowBars" },
															["razUseClassColors"] = { default=false, settingsVal=nil, nControlType=1, strControlName="InCombatNeutralNPCUseClassColors" },
															["razHPBarColor"] 	= { default="ffff00", settingsVal= nil, nControlType=2, strControlName="InCombatNeutralNPCHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatNeutralNPCShowHPText" },
															["razShowResourceBar"]	= { default=false, settingsVal=nil, nControlType=1, strControlName=""},
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatNeutralNPCShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatNeutralNPCTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatNeutralNPCScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="InCombatNeutralNPCOffset" },
														},
				[Unit.CodeEnumDisposition.Friendly] =	{
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyNPCShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyNPCShowGuild" },
															["razNameColor"] ={ default="00ff00", settingsVal= nil, nControlType=2, strControlName="InCombatFriendlyNPCNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyNPCShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyNPCShowBars" },
															["razUseClassColors"] = { default=false, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyNPCUseClassColors" },
															["razHPBarColor"] 	= { default="00ff000", settingsVal= nil, nControlType=2, strControlName="InCombatFriendlyNPCHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyNPCShowHPText" },
															["razShowResourceBar"]	= { default=false, settingsVal=nil, nControlType=1, strControlName=""},
															["razShowCastBar"]		= { default=false, settingsVal=nil, nControlType=1, strControlName="InCombatFriendlyNPCShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatFriendlyNPCTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="InCombatFriendlyNPCScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="InCombatFriendlyNPCOffset" },
														},
			},
	},
	-- out of Combat
	[false] = {
			["MyNameplate"] = {
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatMyNameplateShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatMyNameplateShowGuild" },
															["razNameColor"] = { default="00ff00", settingsVal= nil, nControlType=2, strControlName="OutOfCombatMyNameplateNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatMyNameplateShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatMyNameplateShowBars" },
															["razUseClassColors"] = { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatMyNameplateUseClassColors" },
															["razHPBarColor"] 	= { default="00ff00", settingsVal= nil, nControlType=2, strControlName="OutOfCombatMyNameplateHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatMyNameplateShowHPText" },
															["razShowResourceBar"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatMyNameplateShowResourceBar" },
															["razShowCastBar"]		= { default=false, settingsVal=nil, nControlType=1, strControlName="OutOfCombatMyNameplateShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatMyNameplateTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatMyNameplateScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="OutOfCombatMyNameplateOffset" },

			},
			["TargetNameplate"] = razTargetSettings[false],
			["GroupMember"] = razGroupSettings[false],
			["GuildMember"] =	razGuildSettings[false],
			["Player"]			= {
				[Unit.CodeEnumDisposition.Hostile] = 	{
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostilePlayerShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostilePlayerShowGuild" },
															["razNameColor"] ={ default="ff0000", settingsVal= nil, nControlType=2, strControlName="OutOfCombatHostilePlayerNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostilePlayerShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostilePlayerShowBars" },
															["razUseClassColors"] = { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostilePlayerUseClassColors" },
															["razHPBarColor"] 	= { default="ff0000", settingsVal= nil, nControlType=2, strControlName="OutOfCombatHostilePlayerHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostilePlayerShowHPText" },
															["razShowResourceBar"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostilePlayerShowResourceBar" },
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostilePlayerShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatHostilePlayerTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatHostilePlayerScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="OutOfCombatHostilePlayerOffset" },
														},
				[Unit.CodeEnumDisposition.Friendly] =	{
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyPlayerShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyPlayerShowGuild" },
															["razNameColor"] = { default="00ff00", settingsVal= nil, nControlType=2, strControlName="OutOfCombatFriendlyPlayerNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyPlayerShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyPlayerShowBars" },
															["razUseClassColors"] = { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyPlayerUseClassColors" },
															["razHPBarColor"] 	= { default="00ff00", settingsVal= nil, nControlType=2, strControlName="OutOfCombatFriendlyPlayerHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyPlayerShowHPText" },
															["razShowResourceBar"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyPlayerShowResourceBar" },
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyPlayerShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatFriendlyPlayerTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatFriendlyPlayerScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="OutOfCombatFriendlyPlayerOffset" },
														},						
			},
			["NonPlayer"] 		= {
				[Unit.CodeEnumDisposition.Hostile] =  	{
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostileNPCShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostileNPCShowGuild" },
															["razNameColor"] ={ default="ff0000", settingsVal= nil, nControlType=2, strControlName="OutOfCombatHostileNPCNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostileNPCShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostileNPCShowBars" },
															["razUseClassColors"] = { default=false, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostileNPCUseClassColors" },
															["razHPBarColor"] 	= { default="ff0000", settingsVal= nil, nControlType=2, strControlName="OutOfCombatHostileNPCHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostileNPCShowHPText" },
															["razShowResourceBar"]	= { default=false, settingsVal=nil, nControlType=1, strControlName=""},
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatHostileNPCShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatHostileNPCTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatHostileNPCScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="OutOfCombatHostileNPCOffset" },
														},
				[Unit.CodeEnumDisposition.Neutral] =  	{
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatNeutralNPCShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatNeutralNPCShowGuild" },
															["razNameColor"] ={ default="ffff00", settingsVal= nil, nControlType=2, strControlName="OutOfCombatNeutralNPCNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatNeutralNPCShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatNeutralNPCShowBars" },
															["razUseClassColors"] = { default=false, settingsVal=nil, nControlType=1, strControlName="OutOfCombatNeutralNPCUseClassColors" },
															["razHPBarColor"] 	= { default="ffff00", settingsVal= nil, nControlType=2, strControlName="OutOfCombatNeutralNPCHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatNeutralNPCShowHPText" },
															["razShowResourceBar"]	= { default=false, settingsVal=nil, nControlType=1, strControlName=""},
															["razShowCastBar"]		= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatNeutralNPCShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatNeutralNPCTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatNeutralNPCScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="OutOfCombatNeutralNPCOffset" },
														},
				[Unit.CodeEnumDisposition.Friendly] = 	{
															["razShowName"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyNPCShowName" },
															["razShowGuild"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyNPCShowGuild" },
															["razNameColor"] = { default="00ff00", settingsVal= nil, nControlType=2, strControlName="OutOfCombatFriendlyNPCNameColor" },
															["razShowLevel"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyNPCShowLevel" },
															["razShowBars"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyNPCShowBars" },
															["razUseClassColors"] = { default=false, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyNPCUseClassColors" },
															["razHPBarColor"] 	= { default="00ff00", settingsVal= nil, nControlType=2, strControlName="OutOfCombatFriendlyNPCHPBarColor" },
															["razShowHPText"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyNPCShowHPText" },
															["razShowResourceBar"]	= { default=false, settingsVal=nil, nControlType=1, strControlName=""},
															["razShowCastBar"]		= { default=false, settingsVal=nil, nControlType=1, strControlName="OutOfCombatFriendlyNPCShowCastBar" },
															["razTransparency"] = { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatFriendlyNPCTransparency" },
															["razScale"]		= { default=100, settingsVal=nil, nControlType=3, strControlName="OutOfCombatFriendlyNPCScale" },
															["razOffset"]		= { default=0, settingsVal=nil, nControlType=3, strControlName="OutOfCombatFriendlyNPCOffset" },															
														},
			},
	},
	["GeneralSettings"] = {								--	DoubleCastBarUpdateRate
															["razUseOcclusion"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="UseOcclusion" },
															["razDrawDistance"]	= { default=70, settingsVal=nil, nControlType=3, strControlName="DrawDistanceAll" },
															["razInCombatIfPlayerInCombat"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="InCombatIfPlayerInCombat" },
															["razHideForFriendlySpeechbubble"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="HideForFriendlySpeechbubble" },
															["razShowTargetMarker"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="ShowTargetMarker" },
															["razShowDifficultyIcons"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="ShowDifficultyIcons" },
															["razShowClassIcons"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="ShowClassIcons" },
															["razShowTitles"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="ShowTitles" },
															["razHideUnimportantFriendlyNPCs"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="HideUnimportantFriendlyNPCs" },
															["razMoveDownHugeMobNameplates"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="MoveDownHugeMobNameplates" },
															["razDoubleCastBarUpdateRate"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="DoubleCastBarUpdateRate" },
															["razUseShieldBar"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="ShowShieldBar" },
															["razRemoveShieldBlank"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="RemoveShieldBlank", requiresNot = "ShowShieldBar"  },
															["razNameplateCacheSize"]	= { default=60, settingsVal=nil, nControlType=3, strControlName="NameplateCacheSize" },
															[Unit.CodeEnumDisposition.Friendly] = {
																["razOverrideDrawDistance"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="FriendlyUnitDrawDistanceOverride" },
																["razDrawDistance"]	= { default=70, settingsVal=nil, nControlType=3, strControlName="FriendlyUnitDrawDistance" },
																["razUseOnDamaged"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="FriendlyUnitOnDamaged" },
																["razIncludeShield"] = { default=false, settingsVal=nil, nControlType=1, strControlName="FriendlyUnitOnDamagedCheckShield", requires = "FriendlyUnitOnDamaged" },
																["razShowInterruptArmor"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="FriendlyUnitShowInterruptArmor" },
															},
															[Unit.CodeEnumDisposition.Hostile] = {
																["razOverrideDrawDistance"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="HostileUnitDrawDistanceOverride" },
																["razDrawDistance"]	= { default=70, settingsVal=nil, nControlType=3, strControlName="HostileUnitDrawDistance" },
																["razUseOnDamaged"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="HostileUnitOnDamaged" },
																["razIncludeShield"] = { default=false, settingsVal=nil, nControlType=1, strControlName="HostileUnitOnDamagedCheckShield", requires = "HostileUnitOnDamaged" },
																["razShowInterruptArmor"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="HostileUnitShowInterruptArmor" },
															},
															[Unit.CodeEnumDisposition.Neutral] = {
																["razOverrideDrawDistance"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="NeutralUnitDrawDistanceOverride" },
																["razDrawDistance"]	= { default=70, settingsVal=nil, nControlType=3, strControlName="NeutralUnitDrawDistance" },
																["razUseOnDamaged"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="NeutralUnitOnDamaged" },
																["razIncludeShield"] = { default=false, settingsVal=nil, nControlType=1, strControlName="NeutralUnitOnDamagedCheckShield", requires = "NeutralUnitOnDamaged" },
																["razShowInterruptArmor"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="NeutralUnitShowInterruptArmor" },
															},
	},
	["ColorSettings"] = { 									
															["razShieldBarColor"] = { default="00ffff", settingsVal= nil, nControlType=2, strControlName="ShieldBarColor" },
															["razAbsorbBarColor"] = { default="ffaa00", settingsVal= nil, nControlType=2, strControlName="AbsorbBarColor" },
															["razVulnerableBarColor"] = { default="ff00ff", settingsVal= nil, nControlType=2, strControlName="VulnerableBarColor" },
															["razCastBarColor"] = { default="0000ff", settingsVal= nil, nControlType=2, strControlName="CastBarColor" },
															["razCastBarNoInterColor"] = { default="ff0000", settingsVal= nil, nControlType=2, strControlName="CastBarNoInterColor" },
															["razCastBarOneInterColor"] = { default="ffff00", settingsVal= nil, nControlType=2, strControlName="CastBarOneInterColor" },
															[GameLib.CodeEnumClass.Engineer] = {
																["razClassColor"] = { default="FF9900", settingsVal= nil, nControlType=2, strControlName="ClassColorEngineer" },
															},
															[GameLib.CodeEnumClass.Esper] = {
																["razClassColor"] = { default="00BFFF", settingsVal= nil, nControlType=2, strControlName="ClassColorEsper" },
															},
															[GameLib.CodeEnumClass.Medic] = {
																["razClassColor"] = { default="FFFF00", settingsVal= nil, nControlType=2, strControlName="ClassColorMedic" },
															},
															[GameLib.CodeEnumClass.Spellslinger] = {
																["razClassColor"] = { default="559900", settingsVal= nil, nControlType=2, strControlName="ClassColorSpellslinger" },
															},
															[GameLib.CodeEnumClass.Stalker] = {
																["razClassColor"] = { default="BF00FF", settingsVal= nil, nControlType=2, strControlName="ClassColorStalker" },
															},
															[GameLib.CodeEnumClass.Warrior] = {
																["razClassColor"] = { default="CC0000", settingsVal= nil, nControlType=2, strControlName="ClassColorWarrior" },
															},
	},
	["StyleSettings"] = { 									--RectGFX
															["razMinimalisticMode"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="MinimalisticMode" },
															["razRectGFX"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="RectGFX"},
															["razShieldBarOnTop"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="ShieldBarOnTop" , requires = "ShowShieldBar" },
															["razHPTextCentered"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="HPTextCentered" },
															["razHideCastBarIcon"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="HideCastBarIcon" },
															["razHideEmptyResourceBar"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="HideEmptyResourceBar" },
															["razNameplateWidth"]	= { default=112, settingsVal=nil, nControlType=3, strControlName="NameplateWidth" },
															["razTopBarHeight"]	= { default=25, settingsVal=nil, nControlType=3, strControlName="TopBarHeight" },
															["razMiddleBarHeight"]	= { default=20, settingsVal=nil, nControlType=3, strControlName="MiddleBarHeight" },
															["razBottomBarHeight"]	= { default=20, settingsVal=nil, nControlType=3, strControlName="BottomBarHeight" },
															["razCastBarHeight"]	= { default=25, settingsVal=nil, nControlType=3, strControlName="CastBarHeight" },
															["razHPText1Display"]	= { default=7, settingsVal=nil, nControlType=4, strControlName="HPText1Display", options = razHPTextOptions},
															["razHPText2Display"]	= { default=0, settingsVal=nil, nControlType=4, strControlName="HPText2Display", options = razHPTextOptions},
															["razResourceBarTextDisplay"]	= { default=1, settingsVal=nil, nControlType=4, strControlName="ResourceBarDisplay", options = razResourceBarTextOptions},
															["razNameFont"]	= { default=5, settingsVal=nil, nControlType=4, strControlName="NameFont", options = "GetGameFonts"},
															["razHPTextFont"]	= { default=5, settingsVal=nil, nControlType=4, strControlName="HPTextFont", options = "GetGameFonts"},
															["razResourceBarFont"]	= { default=5, settingsVal=nil, nControlType=4, strControlName="ResourceBarFont", options = "GetGameFonts"},
															["razLevelFont"]	= { default=5, settingsVal=nil, nControlType=4, strControlName="LevelFont", options = "GetGameFonts"},
															["razCastBarFont"]	= { default=5, settingsVal=nil, nControlType=4, strControlName="CastBarFont", options = "GetGameFonts"},
															


	},
	["AlertSettings"] = {
				["ThreatAlert"] =  	{--ThreatAlertInInstanceOnly
															["razThreatAlertEnabled"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="EnableThreatAlert" },
															["razTankModeEnabled"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="TankMode", requires = "EnableThreatAlert" },
															["razThreatAlertInInstanceOnly"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="ThreatAlertInInstanceOnly", requires = "EnableThreatAlert" },
															["razShowThreatMarker"] ={ default=true, settingsVal= nil, nControlType=1, strControlName="ShowThreatMarker", requires = "EnableThreatAlert" },
															["razThreatChangeHPColor"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="ThreatChangeHPColor", requires = "EnableThreatAlert"},
															["razThreatChangeScale"] 	= { default=false, settingsVal= nil, nControlType=1, strControlName="EnableThreatScale", requires = "EnableThreatAlert" },
															["razThreatChangeTransparency"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="EnableThreatTransparency", requires = "EnableThreatAlert"},
															
															["razThreatMarkerColor"]	= { default="ff0000", settingsVal=nil, nControlType=2, strControlName="ThreatMarkerColor", requires = "EnableThreatAlert" },
															["razThreatHPBarColor"] = { default="ff0000", settingsVal=nil, nControlType=2, strControlName="ThreatHPBarColor", requires = "EnableThreatAlert" },
															
															["razThreatScale"]	= { default=0, settingsVal=nil, nControlType=3, strControlName="ThreatScale", requires = "EnableThreatAlert" },
															["razThreatTransparency"] = { default=0, settingsVal=nil, nControlType=3, strControlName="ThreatTransparency", requires = "EnableThreatAlert"},
														
									},
				["LowHPAlert"] =  	{
										[Unit.CodeEnumDisposition.Friendly] =  	{
															["razLowHPAlertEnabled"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="EnableLowHPAlertFriendly" },
															["razLowHPShowMarker"] ={ default=true, settingsVal= nil, nControlType=1, strControlName="ShowLowHPMarkerFriendly", requires = "EnableLowHPAlertFriendly" },
															["razLowHPChangeHPColor"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="LowHPChangeHPColorFriendly" , requires = "EnableLowHPAlertFriendly"},
															["razLowHPChangeScale"] 	= { default=false, settingsVal= nil, nControlType=1, strControlName="EnableLowHPScaleFriendly" , requires = "EnableLowHPAlertFriendly"},
															
															["razLowHPMarkerColor"]	= { default="00ff00", settingsVal=nil, nControlType=2, strControlName="LowHPMarkerColorFriendly", requires = "EnableLowHPAlertFriendly" },
															["razLowHPHPBarColor"] = { default="00ff00", settingsVal=nil, nControlType=2, strControlName="LowHPBarColorFriendly" , requires = "EnableLowHPAlertFriendly"},
															
															["razLowHPScale"]	= { default=0, settingsVal=nil, nControlType=3, strControlName="LowHPScaleFriendly", requires = "EnableLowHPAlertFriendly" },
															["razLowHPThreshold"]	= { default=30, settingsVal=nil, nControlType=3, strControlName="LowHPThresholdFriendly" , requires = "EnableLowHPAlertFriendly"},
															
										},
										[Unit.CodeEnumDisposition.Hostile] =  	{
															["razLowHPAlertEnabled"]	= { default=true, settingsVal=nil, nControlType=1, strControlName="EnableLowHPAlertHostile" },
															["razLowHPShowMarker"] ={ default=true, settingsVal= nil, nControlType=1, strControlName="ShowLowHPMarkerHostile" , requires = "EnableLowHPAlertHostile"},
															["razLowHPChangeHPColor"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="LowHPChangeHPColorHostile", requires = "EnableLowHPAlertHostile" },
															["razLowHPChangeScale"] 	= { default=false, settingsVal= nil, nControlType=1, strControlName="EnableLowHPScaleHostile" , requires = "EnableLowHPAlertHostile"},
															
															["razLowHPMarkerColor"]	= { default="ffff00", settingsVal=nil, nControlType=2, strControlName="LowHPMarkerColorHostile" , requires = "EnableLowHPAlertHostile"},
															["razLowHPHPBarColor"] = { default="ffff00", settingsVal=nil, nControlType=2, strControlName="LowHPBarColorHostile" , requires = "EnableLowHPAlertHostile"},
															
															["razLowHPScale"]	= { default=0, settingsVal=nil, nControlType=3, strControlName="LowHPScaleHostile", requires = "EnableLowHPAlertHostile" },
															["razLowHPThreshold"]	= { default=30, settingsVal=nil, nControlType=3, strControlName="LowHPThresholdHostile", requires = "EnableLowHPAlertHostile" },
										},
									},
			["CleanseAlert"] =  	{
															["razCleanseAlertEnabled"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="EnableCleanseAlert" },
															["razShowCleanseMarker"] ={ default=true, settingsVal= nil, nControlType=1, strControlName="ShowCleanseMarker", requires = "EnableCleanseAlert" },
															["razCleanseChangeHPColor"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="CleanseChangeHPColor", requires = "EnableCleanseAlert"},
															["razCleanseChangeScale"] 	= { default=false, settingsVal= nil, nControlType=1, strControlName="EnableCleanseScale", requires = "EnableCleanseAlert" },
															["razCleanseChangeTransparency"]	= { default=false, settingsVal=nil, nControlType=1, strControlName="EnableCleanseTransparency", requires = "EnableCleanseAlert"},
															
															["razCleanseMarkerColor"]	= { default="9900ff", settingsVal=nil, nControlType=2, strControlName="CleanseMarkerColor", requires = "EnableCleanseAlert" },
															["razCleanseHPBarColor"] = { default="9900ff", settingsVal=nil, nControlType=2, strControlName="CleanseHPBarColor", requires = "EnableCleanseAlert" },
															
															["razCleanseScale"]	= { default=0, settingsVal=nil, nControlType=3, strControlName="CleanseScale", requires = "EnableCleanseAlert" },
														
									},
															
	},
}


-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local karDisposition = 
{
	tTextColors =
	{
		[Unit.CodeEnumDisposition.Hostile] 	= ApolloColor.new("DispositionHostile"),
		[Unit.CodeEnumDisposition.Neutral] 	= ApolloColor.new("DispositionNeutral"),
		[Unit.CodeEnumDisposition.Friendly] = ApolloColor.new("DispositionFriendly"),
	},

	tHealthTextColor =
	{
		[Unit.CodeEnumDisposition.Hostile] 	= "ffff8585",
		[Unit.CodeEnumDisposition.Neutral] 	= "ffffdb57",
		[Unit.CodeEnumDisposition.Friendly] = "ff9bff80",
	},
}
local ktIdToClassSprite =
{
	[GameLib.CodeEnumClass.Esper] 			= "iconEsper",
	[GameLib.CodeEnumClass.Medic] 			= "iconMedic",
	[GameLib.CodeEnumClass.Stalker] 		= "iconStalker",
	[GameLib.CodeEnumClass.Warrior] 		= "iconWarrior",
	[GameLib.CodeEnumClass.Engineer] 		= "iconEngineer",
	[GameLib.CodeEnumClass.Spellslinger] 	= "iconSpellslinger",
}

local ktIdToClassColor =
{
	[GameLib.CodeEnumClass.Engineer] = "ffFF9900",
	[GameLib.CodeEnumClass.Esper] = "ff00BFFF",
	[GameLib.CodeEnumClass.Medic] = "ffFFFF00",
	[GameLib.CodeEnumClass.Spellslinger] = "ff559900",
	[GameLib.CodeEnumClass.Stalker] = "ffBF00FF",
	[GameLib.CodeEnumClass.Warrior] = "ffCC0000"
}

local ktIdToResourceColor =
{
  [GameLib.CodeEnumClass.Engineer] = "ffEE0000",
  [GameLib.CodeEnumClass.Esper] = "ffBE03FD",
  [GameLib.CodeEnumClass.Medic] = "ffBE03FD",
  [GameLib.CodeEnumClass.Spellslinger] = "ffBE03FD",
  [GameLib.CodeEnumClass.Stalker] = "ffFAC205",
  [GameLib.CodeEnumClass.Warrior] = "ffFF3D0D",
  [6] = "000000"
} 
 
local ktHealthBarSprites =
{
	"sprNp_Health_FillGreen",
	"sprNp_Health_FillOrange",
	"sprNp_Health_FillRed"
}

local karConColors =  -- differential value, color
{
	{-4, ApolloColor.new("ConTrivial")},
	{-3, ApolloColor.new("ConInferior")},
	{-2, ApolloColor.new("ConMinor")},
	{-1, ApolloColor.new("ConEasy")},
	{0, ApolloColor.new("ConAverage")},
	{1, ApolloColor.new("ConModerate")},
	{2, ApolloColor.new("ConTough")},
	{3, ApolloColor.new("ConHard")},
	{4, ApolloColor.new("ConImpossible")}
}

local kcrScalingHex 	= "ffffbf80"
local kcrScalingCColor 	= CColor.new(1.0, 191/255, 128/255, 0.7)

local karPathSprite =
{
	[PlayerPathLib.PlayerPathType_Soldier] 		= "CRB_TargetFrameRewardPanelSprites:sprTargetFrame_PathSol",
	[PlayerPathLib.PlayerPathType_Settler] 		= "CRB_TargetFrameRewardPanelSprites:sprTargetFrame_PathSet",
	[PlayerPathLib.PlayerPathType_Scientist] 	= "CRB_TargetFrameRewardPanelSprites:sprTargetFrame_PathSci",
	[PlayerPathLib.PlayerPathType_Explorer] 	= "CRB_TargetFrameRewardPanelSprites:sprTargetFrame_PathExp",
}

local ktDifficultySprites = {
	[Unit.CodeEnumRank.Elite] = "elite",
	[Unit.CodeEnumRank.Superior] = "superior",
	[Unit.CodeEnumRank.Champion] = "champion",
	[Unit.CodeEnumRank.Standard] = "standard",
	[Unit.CodeEnumRank.Minion] = "",
	[Unit.CodeEnumRank.Fodder] = ""
}

local knCharacterWidth 		= 8 -- the average width of a character in the font used. TODO: Not this.
local knRewardWidth 		= 23 -- the width of a reward icon + padding
local knTextHeight 			= 15 -- text window height
local knNameRewardWidth 	= 400 -- the width of the name/reward container
local knNameRewardHeight 	= 20 -- the width of the name/reward container
local knTargetRange 		= 40000 -- the distance^2 that normal nameplates should draw within (max targeting range)
local knNameplatePoolLimit	= 60 -- the window pool max size

-- Todo: break these out onto options
local kcrUnflaggedGroupmate				= ApolloColor.new("DispositionFriendlyUnflaggedDull")
local kcrUnflaggedGuildmate				= ApolloColor.new("DispositionGuildmateUnflagged")
local kcrUnflaggedAlly					= ApolloColor.new("DispositionFriendlyUnflagged")
local kcrFlaggedAlly					= ApolloColor.new("DispositionFriendly")
local kcrUnflaggedEnemyWhenUnflagged 	= ApolloColor.new("DispositionNeutral")
local kcrFlaggedEnemyWhenUnflagged		= ApolloColor.new("DispositionPvPFlagMismatch")
local kcrUnflaggedEnemyWhenFlagged		= ApolloColor.new("DispositionPvPFlagMismatch")
local kcrFlaggedEnemyWhenFlagged		= ApolloColor.new("DispositionHostile")
local kcrDeadColor 						= ApolloColor.new("crayGray")

local kcrDefaultTaggedColor = ApolloColor.new("crayGray")

function RazorPlates:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self

	o.arPreloadUnits = {}
	o.bAddonRestoredOrLoaded = false
	o.SettingsLoaded = false
	o.arWindowPool = {}
	o.arUnit2Nameplate = {}
	o.arWnd2Nameplate = {}
	
	o.bPlayerInCombat = false
	o.guildDisplayed = nil
	o.guildWarParty = nil
	o.nNameplates = 0
	o.nCount = 0
	o.arIterate = {}
	o.bOnRestoreFired = false
    return o
end

function RazorPlates:Init()
	Apollo.RegisterSlashCommand("raz", "OnSlashCommand", self)
    Apollo.RegisterAddon(self, true, nil, {"Tooltips", "RewardIcons"})
end

function RazorPlates:OnSlashCommand(cmd, argument)
	self:OnConfigure()
end

function RazorPlates:OnDependencyError(strDependency, strError)
	return true
end

-----------------------------------------------------------------------------------------------
-- RazorPlates OnLoad
-----------------------------------------------------------------------------------------------

function RazorPlates:OnLoad()
	Apollo.RegisterEventHandler("UnitCreated", 					"OnPreloadUnitCreated", self)
	Apollo.LoadSprites("RAZ.xml")
	self.xmlDoc = XmlDoc.CreateFromFile("RazorPlates.xml")
	--self.xmlDoc:RegisterCallback("OnDocumentReady", self)
	--Apollo.RegisterTimerHandler("LoadTimer", "OnDocumentReady", self)
	--self.timer = ApolloTimer.Create(2, true, "OnTimer", self)

	Apollo.RegisterTimerHandler("PlayerLoadTimer", "WaitForPlayerLoad", self)
	Apollo.CreateTimer("PlayerLoadTimer", 1.0, false); Apollo.StopTimer("PlayerLoadTimer");
	Apollo.RegisterTimerHandler("RazLoadTimer", "WaitForAddonLoad", self)
	Apollo.CreateTimer("RazLoadTimer", 1.0, false); Apollo.StopTimer("RazLoadTimer");

	nWaited = 0
	self:WaitForAddonLoad()
end

function RazorPlates:OnTimer()
	self.unitPlayer = GameLib.GetPlayerUnit()
	--Print("ONTIMER")
	-- self.SettingsLoaded and
	if self.unitPlayer ~= nil then
		self.timer = nil
		self:OnDocumentReady()
	end
end

function RazorPlates:WaitForAddonLoad()
	--Print("Wait for Player")
	nWaited = nWaited + 1
	self.unitPlayer = GameLib.GetPlayerUnit()
	--Print("RUNNING")
	if nWaited > 5 then
		self:OnRestoreNoSaveFile()
	end
	if self.unitPlayer == nil or not self.SettingsLoaded then
		Apollo.StartTimer("RazLoadTimer", 1.0, false)
		return
	end
	--Print("Player Is here")
	--if self.arUnit2Nameplate[self.unitPlayer:GetId()] == nil then
	--	self:OnUnitCreated(self.unitPlayer)
	--end
	self:OnDocumentReady()
end

function RazorPlates:CastBarTimer()
--	for i=1,#self.arIterate do
	--	self:ShowCastBar(self.arIterate[i])
--	end

end

function RazorPlates:PerspectiveResize()
	if self.bPerspectivePlatesDetected then--and self.addonPerspective.settings.perspectiveEnabled then--and self.bUsePerspectivePlates then
		for idx, tNameplate in pairs(self.arUnit2Nameplate) do
	  		Event_FireGenericEvent("GenericEvent_PerspectivePlates_PerspectiveResize",tNameplate)
	 	end
	end
end

function RazorPlates:WaitForPlayerLoad()
	--Print("Wait for Player")
	self.unitPlayer = GameLib.GetPlayerUnit()
	--Print("RUNNING")
	if self.unitPlayer == nil then
		Apollo.StartTimer("PlayerLoadTimer", 1.0, false)
		return
	end
	--Print("Player Is here")
	--if self.arUnit2Nameplate[self.unitPlayer:GetId()] == nil then
	--	self:OnUnitCreated(self.unitPlayer)
	--end
	self:CreateUnitsFromPreload()
end

function RazorPlates:OnPreloadUnitCreated(unitNew)
	self.arPreloadUnits[unitNew:GetId()] = unitNew
end

function RazorPlates.OnOKHook(self)
	local addon = Apollo.GetAddon("RazorPlates")
	--Print("HOOKED YOU")
	addon.OnOKOrg(self)
	if self.settings.perspectiveEnabled then
		addon.curOnFrame = addon.PerspectiveOnFrame
	else
		addon.curOnFrame = addon.NormalOnFrame
		--tNameplate
		for idx, tNameplate in pairs(addon.arUnit2Nameplate) do
	  		self:NameplateRestoreDefaultSize(tNameplate)
	 	end
		--for i=1,#addon.arIterate do
	    	--self:DrawNameplate(self.arIterate[i])
	    --	self:NameplateRestoreDefaultSize(addon.arIterate[i])
	   --	end
	end
	addon:UpdateStyleGlobal()
end

function RazorPlates:OnGroupChanged()
	--for i=1,#self.arIterate do
	    	--self:DrawNameplate(self.arIterate[i])
	--	self:UpdateGroupStatus(self.arIterate[i])
	--end
	--Print("Group Changed")
	for idx, tNameplate in pairs(self.arUnit2Nameplate) do
	  	self:UpdateGroupStatus(tNameplate)
	end
end

function RazorPlates:UnitIsImportantNPC(unitOwner)
	local isImportant = false	
	local tActivation = unitOwner:GetActivationState()
	
	if tActivation.FlightPathSettler ~= nil or tActivation.FlightPath ~= nil or tActivation.FlightPathNew then
		isImportant = true
	end
	if tActivation.Dye ~= nil then
		isImportant = true
	end
	if tActivation.TradeskillTrainer ~= nil then
		isImportant = true
	end
	if tActivation.QuestNewTradeskill ~= nil then
		isImportant = true
	end
	-- QuestGivers too
	if tActivation.QuestReward ~= nil then
		isImportant = true
	end
	if tActivation.QuestNew ~= nil or tActivation.QuestNewMain ~= nil then
		isImportant = true
	end
	if tActivation.QuestReceiving ~= nil then
		isImportant = true
	end
	if tActivation.TalkTo ~= nil then
		isImportant = true
	end
	if tActivation.Vendor ~= nil then
		isImportant = true
	end
	return isImportant	
end

function RazorPlates:NPCIsImportant(tNameplate)
	local unitOwner = tNameplate.unitOwner
	local isImportant = false	
	local tActivation = unitOwner:GetActivationState()
	
	if tActivation.FlightPathSettler ~= nil or tActivation.FlightPath ~= nil or tActivation.FlightPathNew then
		isImportant = true
	end
	if tActivation.Dye ~= nil then
		isImportant = true
	end
	if tActivation.TradeskillTrainer ~= nil then
		isImportant = true
	end
	if tActivation.QuestNewTradeskill ~= nil then
		isImportant = true
	end
	-- QuestGivers too
	if tActivation.QuestReward ~= nil then
		isImportant = true
	end
	if tActivation.QuestNew ~= nil or tActivation.QuestNewMain ~= nil then
		isImportant = true
	end
	if tActivation.QuestReceiving ~= nil then
		isImportant = true
	end
	if tActivation.TalkTo ~= nil then
		isImportant = true
	end
	if tActivation.Vendor ~= nil then
		isImportant = true
	end
	return isImportant	
end


function RazorPlates:OnDocumentReady()
	if self.xmlDoc == nil then
		return
	end
	

	Apollo.RegisterEventHandler("VarChange_FrameCount", 		"OnFrame", self)
	
	Apollo.RegisterEventHandler("UnitTextBubbleCreate", 		"OnUnitTextBubbleToggled", self)
	Apollo.RegisterEventHandler("UnitTextBubblesDestroyed", 	"OnUnitTextBubbleToggled", self)
	Apollo.RegisterEventHandler("TargetUnitChanged", 			"OnTargetUnitChanged", self)
	Apollo.RegisterEventHandler("UnitEnteredCombat", 			"OnEnteredCombat", self)
	Apollo.RegisterEventHandler("UnitNameChanged", 				"OnUnitNameChanged", self)
	Apollo.RegisterEventHandler("UnitTitleChanged", 			"OnUnitTitleChanged", self)
	Apollo.RegisterEventHandler("PlayerTitleChange", 			"OnPlayerTitleChanged", self)
	Apollo.RegisterEventHandler("UnitGuildNameplateChanged", 	"OnUnitGuildNameplateChanged",self)
	Apollo.RegisterEventHandler("UnitLevelChanged", 			"OnUnitLevelChanged", self)
	Apollo.RegisterEventHandler("UnitMemberOfGuildChange", 		"OnUnitMemberOfGuildChange", self)
	Apollo.RegisterEventHandler("GuildChange", 					"OnGuildChange", self)
	Apollo.RegisterEventHandler("UnitGibbed",					"OnUnitGibbed", self)
	Apollo.RegisterEventHandler("ChangeWorld", 					"WaitForPlayerLoad", self)
	--Apollo.RegisterEventHandler("CharacterCreated", 			"WaitForPlayerLoad", self)
	Apollo.RegisterEventHandler("UnitGroupChanged", 								"OnGroupChanged", self)

	local tRewardUpdateEvents = {
		"QuestObjectiveUpdated", "QuestStateChanged", "ChallengeAbandon", "ChallengeLeftArea",
		"ChallengeFailTime", "ChallengeFailArea", "ChallengeActivate", "ChallengeCompleted",
		"ChallengeFailGeneric", "PublicEventObjectiveUpdate", "PublicEventUnitUpdate",
		"PlayerPathMissionUpdate", "FriendshipAdd", "FriendshipPostRemove", "FriendshipUpdate" 
	}
	
	for i, str in pairs(tRewardUpdateEvents) do
		Apollo.RegisterEventHandler(str, "UpdateAllNameplateRewards", self)
	end
	
	self.wndMain = Apollo.LoadForm(self.xmlDoc, "RazorPlatesOptions", nil, self)
	--self.wndHUD = Apollo.LoadForm(self.xmlDoc, "TestWindow", "InWorldHudStratum", self)

	self.wndPlayers = Apollo.LoadForm(self.xmlDoc, "PerUnitTypeModulePlayers", self.wndMain:FindChild("ContentPlayers"), self)
	self.wndNPCs = Apollo.LoadForm(self.xmlDoc, "PerUnitTypeModuleNPCs", self.wndMain:FindChild("ContentNPCs"), self)
	self.wndAlerts = Apollo.LoadForm(self.xmlDoc, "MarkersAndAlertsModule", self.wndMain:FindChild("ContentAlerts"), self)
	self.wndGeneral = Apollo.LoadForm(self.xmlDoc, "GeneralModule", self.wndMain:FindChild("ContentGeneral"), self)
	self.wndColors = Apollo.LoadForm(self.xmlDoc, "ColorsModule", self.wndMain:FindChild("ContentColors"), self)
	self.wndProfiles = Apollo.LoadForm(self.xmlDoc, "ProfilesModule", self.wndMain:FindChild("ContentProfiles"), self)
	self.wndConfirmation = Apollo.LoadForm(self.xmlDoc, "ConfirmationDialog", nil, self)
	self.wndConfirmation:Show(false)
	self.wndMain:Show(false)

	self.wndMain:FindChild("ContentGeneral"):Show(true)
	self.wndMain:FindChild("ContentPlayers"):Show(false)
	self.wndMain:FindChild("ContentNPCs"):Show(false)
	self.wndMain:FindChild("ContentRoles"):Show(false)
	self.wndMain:FindChild("ContentAlerts"):Show(false)
	self.wndMain:FindChild("ContentProfiles"):Show(false)
	self.wndMain:FindChild("ContentColors"):Show(false)

	self.wndMain:FindChild("TabButtonsContainer:GeneralButton"):SetCheck(true)


	Apollo.RegisterTimerHandler("VisibilityTimer", "OnVisibilityTimer", self)
	Apollo.CreateTimer("VisibilityTimer",1.0, true)
	--self.castBarTimer = ApolloTimer.Create(0.1, true, "CastBarTimer", self)
	self:SetRazSettingsDefaults(razPerUnitTypeSettings,false)
	self.arUnit2Nameplate = {}
	self.arWnd2Nameplate = {}
	
	for key, guildCurr in pairs(GuildLib.GetGuilds()) do
		local eGuildType = guildCurr:GetType()
		if eGuildType == GuildLib.GuildType_Guild then
			self.guildDisplayed = guildCurr
		end
		if eGuildType == GuildLib.GuildType_WarParty then
			self.guildWarParty = guildCurr
		end
	end
	
	self.bAllDirty = true
	self.bStyleDirty = true
	self.nFrameCount = 0
	self.nHugeMobs = 0
	-- Cache defaults
	local wndTemp = Apollo.LoadForm(self.xmlDoc, "NameplateNew", nil, self)

	self.nOrgTopBarLeft, self.nOrgTopBarTop, self.nOrgTopBarRight, self.nOrgTopBarBottom = wndTemp:FindChild("RazorContainer:HPandMana:TopBarBack"):GetAnchorOffsets()
	self.nHPFrameTop, self.nHPFrameBottom = 0, 0
	self.nHPFrameHeight = 0

	self.nShieldFrameTop, self.nShieldFrameBottom = 0, 0
	self.nShieldFrameHeight = 0

	self.nResourceFrameTop, self.nResourceFrameBottom = 0, 0
	self.nResourceFrameHeight = 0
	self.nCastFrameTop, self.nCastFrameBottom = 0, 0
	self.nCastFrameHeight = 0

	self.nNameplateWidth = self.nOrgTopBarRight - self.nOrgTopBarLeft
	self.nNameplateLeft = self.nOrgTopBarLeft

	self.nParentLeft, self.nParentTop, self.nParentRight, self.nParentBottom = wndTemp:FindChild("NameplateNew"):GetAnchorOffsets()

	wndTemp:Destroy()
	
	self:UpdateStyleGlobal()
	
	if not razPerUnitTypeSettings["GeneralSettings"].razDoubleCastBarUpdateRate.settingsVal then
		self.curOnFrame = self.NormalOnFrame
	else
		self.curOnFrame = self.SmootherCastBarOnFrame
	end

	Apollo.RemoveEventHandler("UnitCreated", self)
		
	Apollo.RegisterEventHandler("UnitCreated", 					"OnUnitCreated", self)
	Apollo.RegisterEventHandler("UnitDestroyed", 				"OnUnitDestroyed", self)

	self:CreateUnitsFromPreload()

	self.addonPerspective = Apollo.GetAddon("PerspectivePlates")
	self.bPerspectivePlatesDetected = self.addonPerspective ~= nil

	if self.bPerspectivePlatesDetected then
		Event_FireGenericEvent("GenericEvent_PerspectivePlates_RegisterOffsets", -150, -66, 150, 66)
		self.OnOKOrg = self.addonPerspective.OnOK
		self.addonPerspective.OnOK = self.OnOKHook
		if self.addonPerspective.settings.perspectiveEnabled then
			if not razPerUnitTypeSettings["GeneralSettings"].razDoubleCastBarUpdateRate.settingsVal then
				self.curOnFrame = self.PerspectiveOnFrame
			else
				self.curOnFrame = self.SmootherCastBarPerspectiveOnFrame
			end
		else
			if not razPerUnitTypeSettings["GeneralSettings"].razDoubleCastBarUpdateRate.settingsVal then
				self.curOnFrame = self.NormalOnFrame
			else
				self.curOnFrame = self.SmootherCastBarOnFrame
			end
		end
	end
	knNameplatePoolLimit = razPerUnitTypeSettings["GeneralSettings"].razNameplateCacheSize.settingsVal
	self.strCharacterName = GameLib.GetPlayerUnit():GetName()
end

function RazorPlates:OnGeneralView( wndHandler, wndControl, eMouseButton )
	self.wndMain:FindChild("ContentGeneral"):Show(true)
	self.wndMain:FindChild("ContentPlayers"):Show(false)
	self.wndMain:FindChild("ContentNPCs"):Show(false)
	self.wndMain:FindChild("ContentRoles"):Show(false)
	self.wndMain:FindChild("ContentAlerts"):Show(false)
	self.wndMain:FindChild("ContentProfiles"):Show(false)
	self.wndMain:FindChild("ContentColors"):Show(false)

	self.wndMain:FindChild("TabButtonsContainer:GeneralButton"):SetCheck(true)
end

function RazorPlates:OnPlayersView( wndHandler, wndControl, eMouseButton )
	self.wndMain:FindChild("ContentGeneral"):Show(false)
	self.wndMain:FindChild("ContentPlayers"):Show(true)
	self.wndMain:FindChild("ContentNPCs"):Show(false)
	self.wndMain:FindChild("ContentRoles"):Show(false)
	self.wndMain:FindChild("ContentAlerts"):Show(false)
	self.wndMain:FindChild("ContentProfiles"):Show(false)
	self.wndMain:FindChild("ContentColors"):Show(false)

	self.wndMain:FindChild("TabButtonsContainer:PlayersButton"):SetCheck(true)
end

function RazorPlates:OnNPCsView( wndHandler, wndControl, eMouseButton )
	self.wndMain:FindChild("ContentGeneral"):Show(false)
	self.wndMain:FindChild("ContentPlayers"):Show(false)
	self.wndMain:FindChild("ContentNPCs"):Show(true)
	self.wndMain:FindChild("ContentRoles"):Show(false)
	self.wndMain:FindChild("ContentAlerts"):Show(false)
	self.wndMain:FindChild("ContentProfiles"):Show(false)
	self.wndMain:FindChild("ContentColors"):Show(false)

	self.wndMain:FindChild("TabButtonsContainer:NPCsButton"):SetCheck(true)
end

function RazorPlates:OnRolesView( wndHandler, wndControl, eMouseButton )
	self.wndMain:FindChild("ContentGeneral"):Show(false)
	self.wndMain:FindChild("ContentPlayers"):Show(false)
	self.wndMain:FindChild("ContentNPCs"):Show(false)
	self.wndMain:FindChild("ContentRoles"):Show(true)
	self.wndMain:FindChild("ContentAlerts"):Show(false)
	self.wndMain:FindChild("ContentProfiles"):Show(false)
	self.wndMain:FindChild("ContentColors"):Show(false)

	self.wndMain:FindChild("TabButtonsContainer:RolesButton"):SetCheck(true)
end

function RazorPlates:OnMarkersAndAlertsView( wndHandler, wndControl, eMouseButton )
	self.wndMain:FindChild("ContentGeneral"):Show(false)
	self.wndMain:FindChild("ContentPlayers"):Show(false)
	self.wndMain:FindChild("ContentNPCs"):Show(false)
	self.wndMain:FindChild("ContentRoles"):Show(false)
	self.wndMain:FindChild("ContentAlerts"):Show(true)
	self.wndMain:FindChild("ContentProfiles"):Show(false)
	self.wndMain:FindChild("ContentColors"):Show(false)

	self.wndMain:FindChild("TabButtonsContainer:MarkersAndAlertsButton"):SetCheck(true)
end

function RazorPlates:OnProfilesView( wndHandler, wndControl, eMouseButton )
	self.wndMain:FindChild("ContentGeneral"):Show(false)
	self.wndMain:FindChild("ContentPlayers"):Show(false)
	self.wndMain:FindChild("ContentNPCs"):Show(false)
	self.wndMain:FindChild("ContentRoles"):Show(false)
	self.wndMain:FindChild("ContentAlerts"):Show(false)
	self.wndMain:FindChild("ContentProfiles"):Show(true)
	self.wndMain:FindChild("ContentColors"):Show(false)

	self.wndMain:FindChild("TabButtonsContainer:ProfilesButton"):SetCheck(true)
end

function RazorPlates:OnColorsView( wndHandler, wndControl, eMouseButton )
	self.wndMain:FindChild("ContentGeneral"):Show(false)
	self.wndMain:FindChild("ContentPlayers"):Show(false)
	self.wndMain:FindChild("ContentNPCs"):Show(false)
	self.wndMain:FindChild("ContentRoles"):Show(false)
	self.wndMain:FindChild("ContentAlerts"):Show(false)
	self.wndMain:FindChild("ContentProfiles"):Show(false)
	self.wndMain:FindChild("ContentColors"):Show(true)

	self.wndMain:FindChild("TabButtonsContainer:ColorsButton"):SetCheck(true)
end



function RazorPlates:DeepPrintRaz (e)
    -- if e is a table, we should iterate over its elements
    if type(e) == "table" then
        for k,v in pairs(e) do -- for every element in the table
        	if type(k) == "string" and string.find(k, "raz") ~= nil then
           		 Print("Key: " .. tostring(k) .. " Value:" .. tostring(v.settingsVal))
           	else
           		self:DeepPrintRaz(v)  
           	end
           
                 -- recursively repeat the same procedure
        end
    else -- if not, we can just print it
       -- Print(tostring(e))
    end
end

function RazorPlates:DeepPrint (e)
    -- if e is a table, we should iterate over its elements
    if type(e) == "table" then
        for k,v in pairs(e) do -- for every element in the table
        	if type(k) == "string" and string.find(k, "raz") then
        		Print(tostring(k))
        		return
        	end
        	Print(tostring(k) .. "=" .. tostring(v))
        	self:DeepPrint(v)     
                 -- recursively repeat the same procedure
        end
    else -- if not, we can just print it
      --  Print("Single:" .. tostring(e))
    end
end

function RazorPlates:SetRazSettingsDefaults(e, reset)
    if type(e) == "table" then
        for key,val in pairs(e) do -- for every element in the table
           if type(key) == "string" and string.find(key, "raz") ~= nil then
           		if val.settingsVal == nil or reset then
           			val.settingsVal = val.default
           		end
           		if val.nControlType == 1 or val.nControlType == 2 or val.nControlType == 3 or val.nControlType == 4 then
           			local wndControl = self.wndMain:FindChild(val.strControlName)
					if wndControl ~= nil then
						wndControl:SetData(val.strControlName)
					end
           		end
           end
           self:SetRazSettingsDefaults(val, reset)       -- recursively repeat the same procedure
        end
   	end
end

function RazorPlates:SaveRazSettings(e, theDataToSave)
    if type(e) == "table" then
        for key,val in pairs(e) do -- for every element in the table
           if type(key) == "string" and string.find(key, "raz") ~= nil then
           		theDataToSave[val.strControlName] = val.settingsVal
           end
           self:SaveRazSettings(val, theDataToSave)       -- recursively repeat the same procedure
        end
    else -- if not, we can just print it
       -- Print(tostring(e))
    end
end

function RazorPlates:LoadRazSettings(e, theSavedData)
    if type(e) == "table" then
        for key,val in pairs(e) do -- for every element in the table
           if type(key) == "string" and string.find(key, "raz") ~= nil then
           		if theSavedData == nil then
           			val.settingsVal = val.default
           		else
	           		if theSavedData[val.strControlName] ~= nil then
	           			if theSavedData[val.strControlName] ~= nil then
	           				val.settingsVal = theSavedData[val.strControlName]
	           			else
	           				val.settingsVal = val.default
	           			end
	           		end
	           	end
           end
           self:LoadRazSettings(val, theSavedData)       -- recursively repeat the same procedure
        end
	end
end

function RazorPlates:RefreshRazSettings(e)
	 if type(e) == "table" then
        for key,val in pairs(e) do -- for every element in the table
           if type(key) == "string" and string.find(key, "raz") ~= nil then
           		if val.nControlType == 1 then
           			local wndControl = self.wndMain:FindChild(val.strControlName)           			     			
					if val.strControlName ~= "" and wndControl ~= nil then
						wndControl:SetCheck(val.settingsVal)
					end					
				elseif val.nControlType == 2 then
					local wndControl = self.wndMain:FindChild(val.strControlName)
					if val.strControlName ~= "" and wndControl ~= nil then
						wndControl:SetBGColor("ff" .. val.settingsVal)
						wndControl:SetText(val.settingsVal)
					end
				elseif val.nControlType == 3 then
					local wndControl = self.wndMain:FindChild(val.strControlName)
					if val.strControlName ~= "" and wndControl ~= nil then
						wndControl:SetValue(val.settingsVal)
						local info = wndControl:GetPixieInfo(1)
						info.strText = tostring(val.settingsVal)
						wndControl:UpdatePixie(1,info)
					end
				elseif val.nControlType == 4 then
					local wndControl = self.wndMain:FindChild(val.strControlName)
					if val.strControlName ~= "" and wndControl ~= nil then
						if wndControl:GetSelectedText() == nil then
							if val.options ~= "GetGameFonts" then
								for i, s in pairs(val.options) do
									wndControl:AddItem(s)
								end
							else
								for i, s in pairs(razFonts) do
									wndControl:AddItem(s)
								end
							end
						end
						wndControl:SelectItemByIndex(val.settingsVal)
					end
           		end
           end
           self:RefreshRazSettings(val)       -- recursively repeat the same procedure
        end
    end
end

function RazorPlates:RefreshRequirements(e)
	 if type(e) == "table" then
        for key,val in pairs(e) do -- for every element in the table
           if type(key) == "string" and string.find(key, "raz") ~= nil then
           		local wndControl = self.wndMain:FindChild(val.strControlName)
           		          			
				if val.strControlName ~= "" and wndControl ~= nil then
					if val.requires ~= nil then
						local requiredControl = self.wndMain:FindChild(val.requires)
						if 	requiredControl ~= nil then
							wndControl:Enable(requiredControl:IsChecked())
						end
					elseif val.requiresNot ~= nil then
						local notRequiredControl = self.wndMain:FindChild(val.requiresNot)
						if 	notRequiredControl ~= nil then
							wndControl:Enable(not notRequiredControl:IsChecked())
						end
					end
				end
           end
           self:RefreshRequirements(val)       -- recursively repeat the same procedure
        end
    end	
end

function RazorPlates:SetVal(e, key, val)
    if type(e) == "table" then
        for k,v in pairs(e) do -- for every element in the table
        	
           if type(v) == "table" and v.strControlName ~= nil and v.strControlName == key then
           		v.settingsVal = val
           	--	razProfiles[1].Settings[v.strControlName] = val
           	--	Print("Set " .. v.strControlName .. " to: " .. tostring(v.settingsVal))
           		self.bAllDirty = true
           		self.bStyleDirty = true
           		--self:UpdateAllPlates()
           		return
           	else
           		--Print("not key")
           		self:SetVal(v,key,val)
           end
                  -- recursively repeat the same procedure
        end
   	end
end

function RazorPlates:OnSave(eType)
--	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
--		return
--	end
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		local tSave = {}

		--self:SaveRazSettings(razProfiles[1].Settings, tSave[1] )
		--self:SaveRazSettings(razProfiles[2].Settings, tSave[2] )
		self:SaveRazSettings(razPerUnitTypeSettings, tSave)

		return tSave
	end

	if eType == GameLib.CodeEnumAddonSaveLevel.Account then
		local tSave = {}
		--local curProf = tSave[GameLib.GetPlayerUnit:GetName()]
		
		--tSave[GameLib.GetPlayerUnit:GetName()]
		--ProfilesSaved[GameLib.GetPlayerUnit:GetName()]
		--local tSave = ProfilesSaved

		--self:SaveRazSettings(razProfiles[1].Settings, tSave[1] )
		--self:SaveRazSettings(razProfiles[2].Settings, tSave[2] )
		self:SaveRazSettings(razPerUnitTypeSettings, tSave)
		ProfilesSaved[self.strCharacterName] = tSave
		return ProfilesSaved
	end
end

function RazorPlates:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
	--	return
		--end
		self.bOnRestoreFired = true

		local gameFonts = Apollo.GetGameFonts()
		local fonts = {}
		for idx, font in pairs(gameFonts) do
			fonts[idx] = font.name
			razFontSizes[idx] = font.size
		end
		razFonts = fonts

		self:LoadRazSettings(razPerUnitTypeSettings, tSavedData)

		self.SettingsLoaded = true
		
		self:CreateUnitsFromPreload()
	end

	if eType == GameLib.CodeEnumAddonSaveLevel.Account then
		ProfilesSaved = tSavedData
	end
end

function RazorPlates:OnRestoreNoSaveFile()
	if not self.bOnRestoreFired then
		self.bOnRestoreFired = true

		local gameFonts = Apollo.GetGameFonts()
		local fonts = {}
		for idx, font in pairs(gameFonts) do
			fonts[idx] = font.name
			razFontSizes[idx] = font.size
		end
		razFonts = fonts

		self:LoadRazSettings(razPerUnitTypeSettings, nil )

		self.SettingsLoaded = true
		
		self:CreateUnitsFromPreload()
	end
end

function RazorPlates:CreateUnitsFromPreload()
	if self.bAddonRestoredOrLoaded then
		self.unitPlayer = GameLib.GetPlayerUnit()
		-- Process units created while form was loading
		if self.arPreloadUnits == nil then return end
		for idUnit, unitNew in pairs(self.arPreloadUnits) do
			self:OnUnitCreated(unitNew)
		end
		--Print("All Units recreated")
		self.arPreloadUnits = nil
	end
	self.bAddonRestoredOrLoaded = true
end

function RazorPlates:OnVisibilityTimer()
	self:UpdateAllNameplateVisibility()
end

function RazorPlates:UpdateAllNameplateRewards()
	--for idx, tNameplate in pairs(self.arUnit2Nameplate) do
	--for i=1, #self.arIterate do
	--	self:UpdateNameplateRewardInfo(self.arIterate[i])
	--end
	for idx, tNameplate in pairs(self.arUnit2Nameplate) do
	  	self:UpdateNameplateRewardInfo(tNameplate)
	end
end

function RazorPlates:UpdateNameplateRewardInfo(tNameplate)
	local tFlags =
	{
		bVert = false,
		bHideQuests = false,-- not self.bShowRewardTypeQuest,
		bHideChallenges = false,
		bHideMissions = false,
		bHidePublicEvents = false,
		bHideRivals = false,
		bHideFriends = false,
	}
	
	if RewardIcons ~= nil and RewardIcons.GetUnitRewardIconsForm ~= nil then
		RewardIcons.GetUnitRewardIconsForm(tNameplate.wnd.questRewards, tNameplate.unitOwner, tFlags)
	end
end
function RazorPlates:UpdateAllNameplateVisibility()
	--for idx, tNameplate in pairs(self.arUnit2Nameplate) do
	--	self:UpdateNameplateVisibility(tNameplate)
	--end
	--Print("UPDATE")
	for idx, tNameplate in pairs(self.arUnit2Nameplate) do
	  	self:UpdateNameplateVisibility(tNameplate)
	end
end

function RazorPlates:UpdateNameplateVisibility(tNameplate)
	tNameplate.bOnScreen = tNameplate.wndNameplate:IsOnScreen()
	tNameplate.bOccluded = tNameplate.wndNameplate:IsOccluded()
	--tNameplate.eDisposition = tNameplate.unitOwner:GetDispositionTo(self.unitPlayer)
	local bNewShow = self:EvaluateVisibility(tNameplate) and self:CheckDrawDistance(tNameplate)

	--if tNameplate.eDisposition == 2 and not tNameplate.bIsAPlayer and not self:NPCIsImportant(tNameplate) then
	--	bNewShow = false
	--end

	if bNewShow ~= tNameplate.bShow then
	--	if razPerUnitTypeSettings["GeneralSettings"].razHideUnimportantFriendlyNPCs.settingsVal and tNameplate.eDisposition == 2 and not tNameplate.bIsAPlayer and not self:NPCIsImportant(tNameplate) then
	--		bNewShow = false
	--		Print("NOT SO IMPORTANT, HUH? :P")
			--self:DestroyNameplate(tNameplate)
		--HideUnimportantFriendlyNPCs
	--	end
		tNameplate.wndNameplate:Show(bNewShow)
		tNameplate.bShow = bNewShow
		--if bNewShow then
		--	tNameplate.bIsDirty = true
		--end
		tNameplate.bIsDirty = true
	end
end

function RazorPlates:IsGuildMember(tNameplate)
	if not tNameplate.unitOwner:IsThePlayer() then
		local playerGuild = GameLib.GetPlayerUnit():GetGuildName()
		local unitGuild = tNameplate.unitOwner:GetGuildName()
	
		if playerGuild ~= nil and unitGuild ~= nil then
			if playerGuild == unitGuild then
				return true
			end
		end
	end
	return false
end

function RazorPlates:UpdateGroupStatus(tNameplate)
	tNameplate.bIsGuildMember = false
	tNameplate.bIsGroupMember = false
	if tNameplate.unitOwner:IsThePlayer() then
		--tNameplate.bIsGroupMember = true
		return
	end

	if self:IsGuildMember(tNameplate)  then
		tNameplate.bIsGuildMember = true
	end

	if tNameplate.unitOwner:IsInYourGroup() then
		tNameplate.bIsGroupMember = true
	end
	tNameplate.bIsDirty = true
end

function RazorPlates:OnUnitCreated(unitNew) -- build main options here

	if not unitNew:ShouldShowNamePlate() 
		or unitNew:GetType() == "Collectible" 
		or unitNew:GetType() == "PinataLoot" then
		-- Never have nameplates
		--strUnitType == "Pet" or strUnitType == "Esper Pet"
		return
	end

	if razPerUnitTypeSettings["GeneralSettings"].razHideUnimportantFriendlyNPCs.settingsVal then
		if unitNew:GetDispositionTo(self.unitPlayer) == 2 and not ( unitNew:GetType() == "Player") and not self:UnitIsImportantNPC(unitNew) then
			--Print("NOT SO IMPORTANT, HUH?")
			return
		end
	end

	if unitNew:GetDispositionTo(self.unitPlayer) == 3 then
		--Player is not yet loaded - save the units, create later
		if self.arPreloadUnits == nil then
			self.arPreloadUnits = {}
		end
		--Print("Preloading:" .. unitNew:GetName())
		self.arPreloadUnits[unitNew:GetId()] = unitNew
		return
	end

	local idUnit = unitNew:GetId()
	if self.arUnit2Nameplate[idUnit] ~= nil and self.arUnit2Nameplate[idUnit].wndNameplate:IsValid() then
		self.arUnit2Nameplate[idUnit].bIsDirty = true
		return
	end
	
	local wnd = nil
	local wndReferences = nil
	if next(self.arWindowPool) ~= nil then
		local poolEntry = table.remove(self.arWindowPool)
		wnd = poolEntry[1]
		wndReferences = poolEntry[2]
	end
	
	if wnd == nil or not wnd:IsValid() then
		wnd = Apollo.LoadForm(self.xmlDoc, "NameplateNew", "InWorldHudStratum", self)
		--wnd = Apollo.LoadForm(self.xmlDoc, "NameplateNew", self.wndHUD, self)
		wndReferences = nil
	end

	wnd:Show(false, true)
	wnd:SetUnit(unitNew, 1)
	if self.unitPlayer == nil then
		self.unitPlayer = GameLib.GetPlayerUnit()
	end
	local tNameplate =
	{
		unitOwner 		= unitNew,
		idUnit 			= unitNew:GetId(),
		wndNameplate	= wnd,
		bOnScreen 		= wnd:IsOnScreen(),
		bOccluded 		= wnd:IsOccluded(),
		bSpeechBubble 	= false,
		bIsTarget 		= false,
		bWasTarget		= false,
		bIsCluster 		= false,
		bIsCasting 		= false,
		bGibbed			= false,
		--bIsGuildMember 	= self.guildDisplayed and self.guildDisplayed:IsUnitMember(unitNew) or false,
		bIsWarPartyMember = self.guildWarParty and self.guildWarParty:IsUnitMember(unitNew) or false,
		nVulnerableTime = 0,
		eDisposition	= unitNew:GetDispositionTo(self.unitPlayer),
		bShow			= false,
		wnd				= wndReferences,
		unitType		= unitNew:GetType(),
		bIsAPlayer		= unitNew:GetType() == "Player",
		bIsThePlayer	= unitNew:IsThePlayer(),
		classId			= unitNew:GetClassId(),
		bIsDirty		= true,
		lastTarget		= unitNew:GetTarget(),
		bThreatDirty	= false,
		bonusScale		= 0,
		bonusTransparency = 0,
		threatHPColorChange = false,
		LowHPColorChange = false,
		LowHPBonusScale = 0,
		CleanseBonusScale = 0,
		CleanseHPColorChange = false,
		bShowAggroMarker = false,
		bShowLowHPMarker = false,
		bShowCleanseMarker = false,
		nLastHP = 0,
		nLastShield = 0,
		nLastAbsorb = 0,
		nLastResource = -1,
		isDamaged = false,
		bIsShieldDamaged = false,

		nHPBarTop = 0,
		nShieldBarTop = 0,
		nResourceBarTop = 0,
		nCastBarTop = 0,

		Settings = nil,
		NormalSettings = nil,
		--TargetSettings = nil,

		bIsHuge = false,
		bInCombat = false,
		bGotShield = false,
		bIsGuildMember = false,
		bIsGroupMember = false,
	}
	
	local unitType = tNameplate.unitType
	if unitType  == "Player" or unitType == "NonPlayer" then
	else
		unitType = "NonPlayer"
	end
	--tNameplate.Settings[self:InCombatOrDamaged(tNameplate)].razShowBars.settingsVal
	if tNameplate.unitOwner:IsThePlayer() then
		tNameplate.Settings = {
								[true] = razPerUnitTypeSettings[true]["MyNameplate"],
								[false] = razPerUnitTypeSettings[false]["MyNameplate"],
								}
	else
		tNameplate.Settings = {
								[true] = razPerUnitTypeSettings[true][unitType][tNameplate.eDisposition],
								[false] = razPerUnitTypeSettings[false][unitType][tNameplate.eDisposition],
								}
	end							

	--tNameplate.Settings = tNameplate.NormalSettings

	if wndReferences == nil then
		tNameplate.wnd =
		{
			castBar = wnd:FindChild("RazorContainer:HPandMana:CastBar"),
			vulnerable = wnd:FindChild("RazorContainer:HPandMana:Vulnerable"),
			level = wnd:FindChild("Level"),
			guild = wnd:FindChild("Guild"),
			name = wnd:FindChild("NameRewardContainer:Name"),
		--	certainDeath = wnd:FindChild("TargetAndDeathContainer:CertainDeath"),
		--	targetScalingMark = wnd:FindChild("TargetScalingMark"),
			nameRewardContainer = wnd:FindChild("NameRewardContainer:RewardContainer"),
		--	nameReward = wnd:FindChild("NameRewardContainer"),
			hpAndMana = wnd:FindChild("RazorContainer:HPandMana"),


			hpBar = wnd:FindChild("RazorContainer:HPandMana:TopBarFill"),
			hpBarBack = wnd:FindChild("RazorContainer:HPandMana:TopBarBack"),
			hpBarHighlights = wnd:FindChild("RazorContainer:HPandMana:TopBarHighlights"),
			manaBar = wnd:FindChild("RazorContainer:HPandMana:BottomBarFill"),
			manaBarBack = wnd:FindChild("RazorContainer:HPandMana:BottomBarBack"),
			manaBarHighlights = wnd:FindChild("RazorContainer:HPandMana:BottomBarHighlights"),
			shieldBar = wnd:FindChild("RazorContainer:HPandMana:MiddleBarFill"),
			shieldBarBack = wnd:FindChild("RazorContainer:HPandMana:MiddleBarBack"),
			shieldBarHighlights = wnd:FindChild("RazorContainer:HPandMana:MiddleBarHighlights"),
			absorbBar = wnd:FindChild("RazorContainer:HPandMana:MaxAbsorb"),

			castBarText = wnd:FindChild("RazorContainer:HPandMana:CastBarText"),
			castBarNameText = wnd:FindChild("RazorContainer:HPandMana:CastBarNameText"),
			castBarFill = wnd:FindChild("RazorContainer:HPandMana:CastBarFill"),
			castBarBack = wnd:FindChild("RazorContainer:HPandMana:CastBarBack"),
			castBarHighlights = wnd:FindChild("RazorContainer:HPandMana:CastBarHighlights"),
			castBarIcon = wnd:FindChild("RazorContainer:HPandMana:CastBarIcon"),
			vulnerableVulnFill = wnd:FindChild("RazorContainer:HPandMana:Vulnerable:VulnFill"),
			questRewards = wnd:FindChild("NameRewardContainer:RewardContainer:QuestRewards"),
			targetMarker = wnd:FindChild("TargetMarker"),

			classDifficulty = wnd:FindChild("ClassDifficulty"),
			aggroMarker = wnd:FindChild("RazorContainer:HPandMana:aggroMarker"),
			interruptArmor = wnd:FindChild("RazorContainer:HPandMana:InterruptArmor"),
			nameplateParent = wnd:FindChild("RazorContainer"),
			
			hpText1 = wnd:FindChild("RazorContainer:HPandMana:HPText1"),
			hpText2 = wnd:FindChild("RazorContainer:HPandMana:HPText2"),
			hpText3 = wnd:FindChild("RazorContainer:HPandMana:HPText3"),
		}
	end
	if unitType  == "Player" and self:GetSetting(tNameplate,"razUseClassColors") then
		local strColor = strFormat("ff%s", razPerUnitTypeSettings["ColorSettings"][tNameplate.classId].razClassColor.settingsVal)
		tNameplate.wnd.hpBar:SetBGColor(strColor)
	else
		local strColor = strFormat("ff%s",self:GetSetting(tNameplate,"razHPBarColor") )
		tNameplate.wnd.hpBar:SetBGColor(strColor)
	end

	--self.nCount = self.nCount + 1
	--self.arIterate[#self.arIterate+1] = tNameplate

	self.arUnit2Nameplate[idUnit] = tNameplate
	self.arWnd2Nameplate[wnd:GetId()] = tNameplate

--	tNameplate.wnd.shieldBar:Show(false)
	tNameplate.wnd.shieldBarHighlights:Show(false)
	tNameplate.wnd.shieldBarBack:Show(false)
	
	self:UpdateGroupStatus(tNameplate)
	self:UpdateStyle(tNameplate)
	self:UpdateNameplateRewardInfo(tNameplate)
--	self:DrawRewards(tNameplate)
--	self:DrawLowHealth(tNameplate)
	self:DrawThreat(tNameplate)
--	self:ShowGuildAndName(tNameplate)
--	self:ShowLevel(tNameplate)
--	self:ShowClassOrDifficulty(tNameplate)
	self:ShowResourceBar(tNameplate)
--	self:UpdateResourceBar(tNameplate)
--	self:ShowHealthBar(tNameplate)
	
--	self:ShowInterruptArmor(tNameplate)
--	self:DrawTargeting(tNameplate)
--	self:UpdateScaleAndTransparency(tNameplate)
--	self:ShowCastBar(tNameplate)

	--tNameplate.wnd.nameRewardContainer:ArrangeChildrenVert()
end

function RazorPlates:FindAndRemove(tNameplate)
	for i=1, #self.arIterate do
		if self.arIterate[i] == tNameplate then
			table.remove(self.arIterate,i)
		end
	end
end

function RazorPlates:DestroyNameplate(tNameplate)
	if tNameplate ~= nil then
		local wndNameplate = tNameplate.wndNameplate

		self.arWnd2Nameplate[wndNameplate:GetId()] = nil
		wndNameplate:SetNowhere()
		wndNameplate:Show(false, true)
		wndNameplate:SetUnit(nil)

		--self.arWindowPool 
		if self.arWindowPool ~= nil and #self.arWindowPool < knNameplatePoolLimit then
			table.insert(self.arWindowPool, {wndNameplate, tNameplate.wnd})
		else
			wndNameplate:Destroy()
			wndNameplate = nil
		end
	end
end

function RazorPlates:OnUnitDestroyed(unitOwner)
	local idUnit = unitOwner:GetId()
	local tNameplate = self.arUnit2Nameplate[idUnit]
	if tNameplate == nil then
		return
	end
	
	self:DestroyNameplate(tNameplate)
	self.arUnit2Nameplate[idUnit] = nil
end

function RazorPlates:OnFrame()
	self.curOnFrame(self)
end

function RazorPlates:NormalOnFrame()
	self.nFrameCount = self.nFrameCount + 1
	if self.nFrameCount > 1 then

		self.unitPlayer = GameLib.GetPlayerUnit()

	--	for i=1,#self.arIterate do
	 --   	self:DrawNameplate(self.arIterate[i])
	 --   	self:ShowCastBar(self.arIterate[i])
	  -- 	end
	  	for idx, tNameplate in pairs(self.arUnit2Nameplate) do
	  		self:DrawNameplate(tNameplate)
	 		self:ShowCastBar(tNameplate)
	  	end

		self.bAllDirty = false
		self.nFrameCount = 0
	end
end

function RazorPlates:PerspectiveOnFrame()
	self.nFrameCount = self.nFrameCount + 1
	if self.nFrameCount > 1 then

		self.unitPlayer = GameLib.GetPlayerUnit()

		--for i=1,#self.arIterate do
	 --   	self:DrawNameplate(self.arIterate[i])
	  --  	self:ShowCastBar(self.arIterate[i])
	  -- 	end
	  	for idx, tNameplate in pairs(self.arUnit2Nameplate) do
	  		self:DrawNameplate(tNameplate)
	 		self:ShowCastBar(tNameplate)
	  	end

		self.bAllDirty = false
		self.nFrameCount = 0
	end
	self:PerspectiveResize()
end

function RazorPlates:SmootherCastBarOnFrame()
	self.nFrameCount = self.nFrameCount + 1
	if self.nFrameCount > 1 then

		self.unitPlayer = GameLib.GetPlayerUnit()

		--for i=1,#self.arIterate do
	 --   	self:DrawNameplate(self.arIterate[i])
	  --  	self:ShowCastBar(self.arIterate[i])
	  -- 	end
	  	for idx, tNameplate in pairs(self.arUnit2Nameplate) do
	  		self:DrawNameplate(tNameplate)
	 		self:ShowCastBar(tNameplate)
	  	end

		self.bAllDirty = false
		self.nFrameCount = 0
	else
		for idx, tNameplate in pairs(self.arUnit2Nameplate) do
	 		self:ShowCastBar(tNameplate)
	  	end
	end
end

function RazorPlates:SmootherCastBarPerspectiveOnFrame()
	self.nFrameCount = self.nFrameCount + 1
	if self.nFrameCount > 1 then

		self.unitPlayer = GameLib.GetPlayerUnit()

		--for i=1,#self.arIterate do
	 --   	self:DrawNameplate(self.arIterate[i])
	  --  	self:ShowCastBar(self.arIterate[i])
	  -- 	end
	  	for idx, tNameplate in pairs(self.arUnit2Nameplate) do
	  		self:DrawNameplate(tNameplate)
	 		self:ShowCastBar(tNameplate)
	  	end

		self.bAllDirty = false
		self.nFrameCount = 0
	else
		for idx, tNameplate in pairs(self.arUnit2Nameplate) do
	 		self:ShowCastBar(tNameplate)
	  	end
	end
	self:PerspectiveResize()
end


function RazorPlates:InCombatOrDamaged(tNameplate)
	if razPerUnitTypeSettings["GeneralSettings"][tNameplate.eDisposition].razUseOnDamaged.settingsVal then
		return tNameplate.isDamaged
	end
	if razPerUnitTypeSettings["GeneralSettings"].razInCombatIfPlayerInCombat.settingsVal then
		return self.bPlayerInCombat and tNameplate.bInCombat
	else
		return tNameplate.bInCombat
	end
end

function RazorPlates:GetSetting(tNameplate, settingName)
	local combatState = self:InCombatOrDamaged(tNameplate)

	if not tNameplate.bIsTarget then
		if tNameplate.bIsGroupMember then
			return razGroupSettings[combatState][settingName].settingsVal
		elseif tNameplate.bIsGuildMember then
			return razGuildSettings[combatState][settingName].settingsVal
		end
		return tNameplate.Settings[combatState][settingName].settingsVal
	else
		if razTargetSettings[combatState][settingName] == nil then
			if tNameplate.bIsGroupMember then
				return razGroupSettings[combatState][settingName].settingsVal
			elseif tNameplate.bIsGuildMember then
				return razGuildSettings[combatState][settingName].settingsVal
			end
			return tNameplate.Settings[combatState][settingName].settingsVal
		else
			return razTargetSettings[combatState][settingName].settingsVal
		end
	end
end


function RazorPlates:DrawNameplate(tNameplate)
	if not tNameplate.bShow and not self.bAllDirty and not tNameplate.bIsDirty then
		return
	end

	local unitOwner = tNameplate.unitOwner
	local wndNameplate = tNameplate.wndNameplate
	
	if unitOwner:IsMounted() and wndNameplate:GetUnit() == unitOwner then
		wndNameplate:SetUnit(unitOwner:GetUnitMount(), 1)
	elseif not unitOwner:IsMounted() and wndNameplate:GetUnit() ~= unitOwner then
		wndNameplate:SetUnit(unitOwner, 1)
	end

	local curTarget = unitOwner:GetTarget()
	if curTarget ~= tNameplate.lastTarget then
		tNameplate.lastTarget = curTarget
		tNameplate.bThreadDirty = true
	end

	if tNameplate.bThreadDirty then
		self:GetSetting(tNameplate,"razShowBars")
		self:DrawThreat(tNameplate)
		self:UpdateScaleAndTransparency(tNameplate)
		tNameplate.bThreadDirty = false
	end

	if self.bStyleDirty then
		self:UpdateStyleGlobal()
		self.bStyleDirty = false
	end

	if self.bAllDirty then
		self:UpdateStyle(tNameplate)
	end
	--Print(tNameplate.unitOwner:GetDispositionTo(self.unitPlayer))
	local newDispo = tNameplate.unitOwner:GetDispositionTo(self.unitPlayer)
	if tNameplate.eDisposition ~= newDispo then
		--Print(tostring(unitOwner:GetType()) .. tostring(tNameplate.eDisposition) .. " to " .. tostring(newDispo) )
		
		if tNameplate.unitType == "NonPlayer" then
			tNameplate.eDisposition = newDispo
			--Print("Updating Settings")
			self:UpdateSettingsPointer(tNameplate)
			tNameplate.bIsDirty = true
		end
	end

	self:DrawTargeting(tNameplate)
	self:ShowHealthBar(tNameplate)

	if self.bAllDirty or tNameplate.bIsDirty then 
		--self:UpdateGroupStatus(tNameplate)
		--self:DrawTargeting(tNameplate)
		if tNameplate.bIsAPlayer then
			self:UpdateStyle(tNameplate)
		end
		self:ShowGuildAndName(tNameplate)
		self:ShowLevel(tNameplate)
		self:ShowClassOrDifficulty(tNameplate)
	--	self:UpdateHPAndShieldBar(tNameplate)
		--self:UpdateResourceBar(tNameplate)
		--self:ShowInterruptArmor(tNameplate)
		--self:DrawRewards(tNameplate)
--		self:ShowHealthBar(tNameplate)
--		self:DrawLowHealth(tNameplate)
--		self:DrawTargeting(tNameplate)
		self:ShowResourceBar(tNameplate)
		self:UpdateScaleAndTransparency(tNameplate)
		tNameplate.bIsDirty = false
	end

	--self:ShowCastBar(tNameplate)
	self:ShowInterruptArmor(tNameplate)
	--self:UpdateNameplateRewardInfo(tNameplate)
	self:DrawRewards(tNameplate)
	
	self:DrawLowHealth(tNameplate)
	self:UpdateResourceBar(tNameplate)
	self:MoveDownHugeEnemyNameplate(tNameplate)
	self:DrawCleanse(tNameplate)
end

function RazorPlates:MoveDownHugeEnemyNameplate(tNameplate)
	if not razPerUnitTypeSettings["GeneralSettings"].razMoveDownHugeMobNameplates.settingsVal or tNameplate.bIsAPlayer or not tNameplate.bInCombat or not self.bPlayerInCombat or not tNameplate.bOnScreen or tNameplate.eDisposition == 2 then
		if tNameplate.bIsHuge then
			local wndNameplate = tNameplate.wndNameplate
			wndNameplate:SetAnchorOffsets(-150, -66, 150, 66)
			tNameplate.bIsHuge = false
		end
    	return
	end
	local unitOwner = tNameplate.unitOwner
	local wndNameplate = tNameplate.wndNameplate
	--Print(unitOwner:GetName() .. ":" ..tostring(tNameplate.bIsHuge))
	local unitAnchor = unitOwner:GetOverheadAnchor() 
	local playerWnd = self.arUnit2Nameplate[self.unitPlayer:GetId()].wndNameplate
	if unitAnchor ~= nil then
		--Print(unitOwner:GetName() .. ":" ..unitAnchor.y)
		local offset = unitAnchor.y -66
		local posX,posY =  wndNameplate:GetPos()
		local posX2, posY2 =  playerWnd:GetPos()

		if offset < 0 then
			--Print(unitOwner:GetName() .. ":" .. posX .. ":".. posY)
			--Print("Dhyzk:" .. posX2 .. ":".. posY2)
			local xOffset = ( posX2 - posX ) * 0.25
			--if not tNameplate.bIsHuge then
				wndNameplate:SetUnit(self.unitPlayer, 1)
				--if self.nHugeMobs == 0 then
				--	wndNameplate:SetAnchorOffsets(-450, -66-122, -150, 66-122)
				--elseif self.nHugeMobs == 1 then
				wndNameplate:SetAnchorOffsets(-150-xOffset, -66-122, 150-xOffset, 66-122)
			--	elseif self.nHugeMobs == 2 then
			--		wndNameplate:SetAnchorOffsets(150, -66-122, 450, 66-122)
			--	end
				tNameplate.bIsHuge = true
			--end
			--self.nHugeMobs = self.nHugeMobs + 1
		else
			wndNameplate:SetAnchorOffsets(-150, -66, 150, 66)
			tNameplate.bIsHuge = false
		end
	end
end

function RazorPlates:ShowHealthBar(tNameplate)
	if tNameplate.unitOwner:GetHealth() == nil then
			--tNameplate.wnd.hpAndMana:Show(false)
			tNameplate.wnd.hpBar:Show(false)
			tNameplate.wnd.hpBarBack:Show(false)
			tNameplate.wnd.hpBarHighlights:Show(false)
			tNameplate.wnd.aggroMarker:Show(false)
			tNameplate.wnd.hpText1:SetText("")
			tNameplate.wnd.hpText2:SetText("")
		return
	end
	local bShowHealthBar = self:GetSetting(tNameplate,"razShowBars") -- tNameplate.Settings[self:InCombatOrDamaged(tNameplate)].razShowBars.settingsVal
	--Print(tNameplate.unitOwner:GetName() .. ":" .. tostring(bShowHealthBar))
	local aggroMarker = tNameplate.wnd.aggroMarker

	if bShowHealthBar ~= tNameplate.wnd.hpBar:IsShown() then
	--	tNameplate.wnd.hpAndMana:Show(bShowHealthBar)
		tNameplate.wnd.hpBar:Show(bShowHealthBar)
		tNameplate.wnd.hpBarBack:Show(bShowHealthBar)
		tNameplate.wnd.hpBarHighlights:Show(bShowHealthBar and not razPerUnitTypeSettings["StyleSettings"].razMinimalisticMode.settingsVal)

		tNameplate.wnd.shieldBar:Show(bShowHealthBar)
		tNameplate.wnd.shieldBarBack:Show(bShowHealthBar)
		tNameplate.wnd.shieldBarHighlights:Show(bShowHealthBar and not razPerUnitTypeSettings["StyleSettings"].razMinimalisticMode.settingsVal)
		tNameplate.wnd.absorbBar:Show(bShowHealthBar)
	end

	if (tNameplate.bShowAggroMarker or tNameplate.bShowLowHPMarker or tNameplate.bShowCleanseMarker) ~= aggroMarker:IsShown() then
		aggroMarker:Show(tNameplate.bShowAggroMarker or tNameplate.bShowLowHPMarker or tNameplate.bShowCleanseMarker)
	end

	if not tNameplate.wnd.hpBar:IsShown() then
		aggroMarker:Show(false)
	end

	if bShowHealthBar then
		local wndVulnerable = tNameplate.wnd.vulnerable
		local unitType =tNameplate.unitType

		
		local nVulnerabilityTime = tNameplate.unitOwner:GetCCStateTimeRemaining(Unit.CodeEnumCCState.Vulnerability)

		if nVulnerabilityTime and nVulnerabilityTime > 0 then
			local wndVulnerable = tNameplate.wnd.vulnerable
			local strColor = strFormat("ff%s", razPerUnitTypeSettings["ColorSettings"].razVulnerableBarColor.settingsVal)
			tNameplate.wnd.hpBar:SetBGColor(strColor)
			if nVulnerabilityTime == 0 and nVulnerabilityTime ~= tNameplate.nVulnerableTime then
				nVulnerabilityTime = 0 -- casting done, set back to 0
				wndVulnerable:Show(false)
			elseif nVulnerabilityTime ~= 0 and nVulnerabilityTime > tNameplate.nVulnerableTime then
				tNameplate.nVulnerableTime = nVulnerabilityTime
				wndVulnerable:Show(true)
			--	bIsVulnerable = true
			elseif nVulnerabilityTime ~= 0 and nVulnerabilityTime < tNameplate.nVulnerableTime then
				tNameplate.wnd.vulnerableVulnFill:SetMax(tNameplate.nVulnerableTime)
				tNameplate.wnd.vulnerableVulnFill:SetProgress(nVulnerabilityTime)
			--	bIsVulnerable = true
			end
		else
			wndVulnerable:Show(false)

			if tNameplate.LowHPColorChange then
				eDisposition = tNameplate.eDisposition
				if eDisposition == 1 then
					eDisposition = 0
				end
				local strColor = strFormat("ff%s", razPerUnitTypeSettings["AlertSettings"]["LowHPAlert"][eDisposition].razLowHPHPBarColor.settingsVal)
				tNameplate.wnd.hpBar:SetBGColor(strColor)
			elseif tNameplate.threatHPColorChange then
				local strColor = strFormat("ff%s", razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatHPBarColor.settingsVal)
				tNameplate.wnd.hpBar:SetBGColor(strColor)
			elseif tNameplate.CleanseHPColorChange then 
				local strColor = strFormat("ff%s", razPerUnitTypeSettings["AlertSettings"]["CleanseAlert"].razCleanseHPBarColor.settingsVal)
				tNameplate.wnd.hpBar:SetBGColor(strColor)
			else
				if unitType  == "Player" and self:GetSetting(tNameplate,"razUseClassColors") then
					local strColor = strFormat("ff%s", razPerUnitTypeSettings["ColorSettings"][tNameplate.classId].razClassColor.settingsVal)
					tNameplate.wnd.hpBar:SetBGColor(strColor)
				else
					local strColor = strFormat("ff%s",self:GetSetting(tNameplate,"razHPBarColor") )
					tNameplate.wnd.hpBar:SetBGColor(strColor)
				end
			end			
		end
	end
	self:UpdateHPAndShieldBar(tNameplate, bShowHealthBar)
end

function RazorPlates:UpdateHPAndShieldBar(tNameplate,showHPBar)
	local unitType = tNameplate.unitType
	local unitOwner = tNameplate.unitOwner

	if tNameplate.unitType == "Simple" or unitOwner:GetHealth() == nil then
		if showHPBar then
			tNameplate.wnd.hpBar:SetAnchorOffsets(self.nNameplateLeft+2, tNameplate.nHPBarTop+2, self.nNameplateLeft + self.nNameplateWidth-2,tNameplate.nHPBarTop + self.nHPFrameHeight-1)
			--Print(unitOwner.GetName() .. " is just that simple..")
			local strColor = strFormat("ff%s",self:GetSetting(tNameplate,"razHPBarColor") )
			tNameplate.wnd.hpBar:SetBGColor(strColor)
			--tNameplate.wnd.healthHealthLabel:SetText("")
		end
		tNameplate.wnd.hpText1:SetText("")
		tNameplate.wnd.hpText2:SetText("")
		return
	end

	local nHealthCurr 	= unitOwner:GetHealth()
	local nHealthMax 	= unitOwner:GetMaxHealth()
	local nShieldCurr 	= unitOwner:GetShieldCapacity()
	local nShieldMax 	= unitOwner:GetShieldCapacityMax()
	
	if nShieldMax > 0 then
		tNameplate.bGotShield = true
	else
		tNameplate.bGotShield = false
	end

	local nAbsorbCurr = 0
	local nAbsorbMax = unitOwner:GetAbsorptionMax()
	if nAbsorbMax > 0 then
		nAbsorbCurr = unitOwner:GetAbsorptionValue() -- Since it doesn't clear when the buff drops off
	end

	if unitOwner:IsDead() then
		nHealthCurr = 0
	end

	
	if tNameplate.nLastHP == nHealthCurr and tNameplate.nLastShield == nShieldCurr and tNameplate.nLastAbsorb == nAbsorbCurr and not self.bAllDirty and not tNameplate.bIsDirty then
		--Print("Fucking the same!")
		return
	end

	local newIsDamaged =  nHealthCurr < nHealthMax or (nShieldCurr < nShieldMax and razPerUnitTypeSettings["GeneralSettings"][tNameplate.eDisposition].razIncludeShield.settingsVal)

	if tNameplate.isDamaged ~= newIsDamaged then
		tNameplate.isDamaged = newIsDamaged
		tNameplate.bIsDirty = true
		self:ShowHealthBar(tNameplate)
		return
	end

	tNameplate.nLastHP = nHealthCurr
	tNameplate.nLastShield = nShieldCurr 
	tNameplate.nLastAbsorb = nAbsorbCurr

	if showHPBar then
		if razPerUnitTypeSettings["GeneralSettings"].razUseShieldBar.settingsVal then 

			if nShieldMax < nShieldCurr then nShieldMax = nShieldCurr end

			local nPointHealthRight = self.nNameplateLeft + ((self.nNameplateWidth) * (nHealthCurr / (nHealthMax+nAbsorbCurr) )) -- applied to the difference between L and R
			local nPointShieldRight = self.nNameplateLeft + ((self.nNameplateWidth) * ((nShieldCurr) / nShieldMax)) 
			local nPointAbsorbLeft = self.nNameplateLeft + (self.nNameplateWidth * ((nHealthCurr ) / (nHealthMax+nAbsorbCurr)))
			local nPointAbsorbRight = self.nNameplateLeft + (self.nNameplateWidth * ((nHealthCurr + nAbsorbCurr) / (nHealthMax+nAbsorbCurr))) 
			
			if showHPBar then
				tNameplate.wnd.hpBar:SetAnchorOffsets(self.nNameplateLeft+2, tNameplate.nHPBarTop+2, nPointHealthRight-3, tNameplate.nHPBarTop + self.nHPFrameHeight-1)
				tNameplate.wnd.absorbBar:SetAnchorOffsets(nPointAbsorbLeft-2, tNameplate.nHPBarTop+2, nPointAbsorbRight-3,tNameplate.nHPBarTop + self.nHPFrameHeight-1)
				
				tNameplate.wnd.absorbBar:Show(nHealthCurr > 0 and nAbsorbCurr > 0)
			end
			--if nShieldMax > 0 and nShieldCurr > 0 then
			tNameplate.wnd.shieldBar:SetAnchorOffsets(self.nNameplateLeft+2, tNameplate.nShieldBarTop+2, nPointShieldRight-3, tNameplate.nShieldBarTop + self.nShieldFrameHeight-1)
			--end
			--if nAbsorbMax > 0 and nAbsorbCurr > 0 then
				
			--end
			--end
			tNameplate.wnd.shieldBar:Show(nHealthCurr > 0 and nShieldCurr > 0)
			tNameplate.wnd.shieldBarBack:Show(nHealthCurr > 0 and nShieldMax > 0)
			tNameplate.wnd.shieldBarHighlights:Show(not razPerUnitTypeSettings["StyleSettings"].razMinimalisticMode.settingsVal and nHealthCurr > 0 and nShieldMax > 0 )
				
		else
			--if showHPBar then
				if razPerUnitTypeSettings["GeneralSettings"].razRemoveShieldBlank.settingsVal then
					nShieldMax = nShieldCurr
					nAbsorbMax = nAbsorbCurr
				end
				local nTotalMax = nHealthMax + nShieldMax + nAbsorbMax
			
				-- Scaling
				local nPointHealthRight = self.nNameplateLeft + ((self.nNameplateWidth) * (nHealthCurr / nTotalMax)) -- applied to the difference between L and R
				local nPointShieldRight = self.nNameplateLeft + ((self.nNameplateWidth) * ((nHealthCurr + nShieldCurr) / nTotalMax))
				local nPointAbsorbLeft = self.nNameplateLeft + (self.nNameplateWidth * ((nHealthCurr + nShieldMax) / nTotalMax))
				local nPointAbsorbRight = self.nNameplateLeft + (self.nNameplateWidth * ((nHealthCurr + nShieldMax + nAbsorbMax) / nTotalMax))
			
				-- Resize
				tNameplate.wnd.hpBar:SetAnchorOffsets(self.nNameplateLeft+2, tNameplate.nHPBarTop+2, nPointHealthRight-3, tNameplate.nHPBarTop + self.nHPFrameHeight-1)

				--if nShieldMax > 0 and nShieldCurr > 0 then
					tNameplate.wnd.shieldBar:SetAnchorOffsets(nPointHealthRight-6, tNameplate.nHPBarTop+2, nPointShieldRight-3, tNameplate.nHPBarTop + self.nHPFrameHeight-1)
				--end
				--if nAbsorbMax > 0 and nAbsorbCurr > 0 then
					tNameplate.wnd.absorbBar:SetAnchorOffsets(nPointAbsorbLeft-6, tNameplate.nHPBarTop+2, nPointAbsorbRight-3, tNameplate.nHPBarTop + self.nHPFrameHeight-1)
				--end
				tNameplate.wnd.shieldBarBack:Show(false)
				tNameplate.wnd.shieldBarHighlights:Show(false )

				tNameplate.wnd.shieldBar:Show(nHealthCurr > 0 and nShieldCurr > 0)
				tNameplate.wnd.absorbBar:Show(nHealthCurr > 0 and nAbsorbCurr > 0)
			--end
		end

	end
	if self:GetSetting(tNameplate,"razShowHPText")  then
			tNameplate.wnd.hpText1:SetText(self:GetHPText(nHealthCurr,nHealthMax, nShieldCurr, razPerUnitTypeSettings["StyleSettings"].razHPText1Display.settingsVal))
			tNameplate.wnd.hpText2:SetText(self:GetHPText(nShieldCurr,nShieldMax, nShieldCurr, razPerUnitTypeSettings["StyleSettings"].razHPText2Display.settingsVal))
	else
			tNameplate.wnd.hpText1:SetText("")
			tNameplate.wnd.hpText2:SetText("")
	end

end

function RazorPlates:UpdateResourceBar(tNameplate)
	if not tNameplate.bIsAPlayer then tNameplate.wnd.hpText3:SetText("") return end
	local nManaCurr, nManaMax = self:GetClassResource(tNameplate.unitOwner)
	if tNameplate.nLastResource == nManaCurr and not self.bAllDirty and not tNameplate.bIsDirty then
		--Print("Fucking the same!")
		return
	end
	tNameplate.nLastResource = nManaCurr
	if nManaMax ~= nil then
		if nManaMax == 0 then
				nManaMax = 1000
		end
		local nPointManaRight = self.nNameplateLeft + ((self.nNameplateWidth) * (nManaCurr / nManaMax))

		tNameplate.wnd.manaBar:SetAnchorOffsets(self.nNameplateLeft+2, tNameplate.nResourceBarTop+2, nPointManaRight-3 , tNameplate.nResourceBarTop + self.nResourceFrameHeight-2)
	end
	if self:GetSetting(tNameplate,"razShowHPText") then
		--tNameplate.wnd.hpText1:SetText(self:GetCornerTextStr(tNameplate,razPerUnitTypeSettings["StyleSettings"].razResourceBarTextDisplay.settingsVal))
		tNameplate.wnd.hpText3:SetText(self:GetHPText(nManaCurr,nManaMax,0,razPerUnitTypeSettings["StyleSettings"].razResourceBarTextDisplay.settingsVal))
	else
		--tNameplate.wnd.hpText1:SetText("")
		tNameplate.wnd.hpText3:SetText("")
	end
	--tNameplate.wnd.manaBarBack:Show( not (nManaCurr == 0 and razPerUnitTypeSettings["StyleSettings"].razHideEmptyResourceBar.settingsVal))
	--tNameplate.wnd.manaBarHighlights:Show( not (nManaCurr == 0 and razPerUnitTypeSettings["StyleSettings"].razHideEmptyResourceBar.settingsVal))

end

function RazorPlates:GetAbilitiesList()
	if self.abilitiesList == nil then
		self.abilitiesList = AbilityBook.GetAbilitiesList()
	end
	return self.abilitiesList
end

function RazorPlates:GetSpellIcon(spellName)
	local abilities = self:GetAbilitiesList()
	if abilities ~= nil then
		for _, ability in pairs(abilities) do
			if ability.strName == spellName then
				return ability.tTiers[1].splObject:GetIcon()
			end
		end
	end
	return ""
end

function RazorPlates:ShowCastBar(tNameplate)
	if not tNameplate.bShow or not self:GetSetting(tNameplate,"razShowCastBar") or not tNameplate.unitOwner:ShouldShowCastBar() then
		if tNameplate.wnd.castBarBack:IsShown() then
			tNameplate.wnd.castBarBack:Show(false)
			tNameplate.wnd.castBarFill:Show(false)
			tNameplate.wnd.castBarHighlights:Show(false)
			tNameplate.wnd.castBarIcon:Show(false)
			tNameplate.wnd.castBarNameText:SetText("")
			tNameplate.wnd.castBarText:SetText("")
		end
		return
	end
	
	if not tNameplate.wnd.castBarBack:IsShown() then
		tNameplate.wnd.castBarIcon:Show(not razPerUnitTypeSettings["StyleSettings"].razHideCastBarIcon.settingsVal)
		tNameplate.wnd.castBarBack:Show(true)
		tNameplate.wnd.castBarFill:Show(true)
		tNameplate.wnd.castBarHighlights:Show(not razPerUnitTypeSettings["StyleSettings"].razMinimalisticMode.settingsVal)

		--tNameplate.wnd.castBarBack:SetAnchorOffsets(0,self.nCastTop-2,0, self.nCastTop - 2 + razPerUnitTypeSettings["StyleSettings"].razCastBarHeight.settingsVal)
		local strSpellName = tNameplate.unitOwner:GetCastName()
		tNameplate.wnd.castBarNameText:SetText(strSpellName)
		tNameplate.wnd.castBarFill:SetMax(tNameplate.unitOwner:GetCastDuration())
		if not razPerUnitTypeSettings["StyleSettings"].razHideCastBarIcon.settingsVal and strSpellName ~= "SpellName" and strSpellName ~= "" then
			tNameplate.wnd.castBarIcon:SetSprite( self:GetSpellIcon(strSpellName) )
		end
	end
	local nArmor = tNameplate.unitOwner:GetInterruptArmorValue() or 0
	local nArmorMax = tNameplate.unitOwner:GetInterruptArmorMax()
	if nArmorMax == -1 or nArmor > 1 then
		local strColor = strFormat("ff%s", razPerUnitTypeSettings["ColorSettings"].razCastBarNoInterColor.settingsVal)
		tNameplate.wnd.castBarFill:SetBarColor(strColor)
	elseif nArmor == 1 then
		local strColor = strFormat("ff%s", razPerUnitTypeSettings["ColorSettings"].razCastBarOneInterColor.settingsVal)
		tNameplate.wnd.castBarFill:SetBarColor(strColor)
	else
		local strColor = strFormat("ff%s", razPerUnitTypeSettings["ColorSettings"].razCastBarColor.settingsVal)
		tNameplate.wnd.castBarFill:SetBarColor(strColor)
	end
		--Print("Casting: " .. unitOwner:GetCastName())
	
	tNameplate.wnd.castBarFill:SetProgress(tNameplate.unitOwner:GetCastElapsed())
	local strText = strFormat("%.1fs", (tNameplate.unitOwner:GetCastDuration() - tNameplate.unitOwner:GetCastElapsed()) * 0.001)
	tNameplate.wnd.castBarText:SetText(strText)
end

function RazorPlates:GetHPText(cur, max, curShield, ctextoption)
	local strText = ""
	if ctextoption == 1 then --cur
		if cur > 0 then strText = self:FormatBigNumber(cur) end
	elseif ctextoption == 2 then --cur/max
		if max > 0 then strText = strFormat("%s/%s",self:FormatBigNumber(cur),self:FormatBigNumber(max)) end
	elseif ctextoption == 3 then --cur%
		if max > 0 then strText =  strFormat("%d%%",math.floor(cur / max *100)) end
	elseif ctextoption == 4 then --deficit
		local deficit = cur - max
		if deficit < 0 then strText = self:FormatBigNumber(deficit) end
	elseif ctextoption == 5 then 
		if max > 0 then strText = strFormat("%s/%s-%s",self:FormatBigNumber(cur),self:FormatBigNumber(max),strFormat("%d%%",math.floor(cur / max *100))) end
	elseif ctextoption == 6 then 
		if max > 0 then strText = strFormat("%s-%s",self:FormatBigNumber(cur),strFormat("%d%%",math.floor(cur / max *100))) end
	elseif ctextoption == 7 then
		if max > 0 then
			if curShield > 0 then
				strText = strFormat("%s/%s (%s)",self:FormatBigNumber(cur),self:FormatBigNumber(max),self:FormatBigNumber(curShield))
			else
				strText = strFormat("%s/%s",self:FormatBigNumber(cur),self:FormatBigNumber(max))
			end
		end
	elseif ctextoption == 8 then
		if max > 0 then
			if curShield > 0 then
				strText = strFormat("%s (%s)",self:FormatBigNumber(cur),self:FormatBigNumber(curShield))
			else
				strText = strFormat("%s",self:FormatBigNumber(cur))
			end
		end
	end
	return strText
end

function RazorPlates:ShowGuildAndName(tNameplate)
	local unitOwner = tNameplate.unitOwner
	local wndGuild = tNameplate.wnd.guild
	local wndName = tNameplate.wnd.name
	
	local bShowName =  self:GetSetting(tNameplate,"razShowName")
	local strNewName = ""

	local strColor = strFormat("ff%s", self:GetSetting(tNameplate,"razNameColor"))
	wndName:SetTextColor(strColor)
	wndGuild:SetTextColor(strColor)

	if bShowName then
		if razPerUnitTypeSettings["GeneralSettings"].razShowTitles.settingsVal then
			strNewName = unitOwner:GetTitleOrName()
		else
			strNewName = unitOwner:GetName()
		end
		--if wndName:GetText() ~= strNewName then
		--	wndName:SetText(strNewName)
		--end
	end
	
	
	local strNewGuild = unitOwner:GetAffiliationName()
	if tNameplate.bIsAPlayer and strNewGuild ~= nil and strNewGuild ~= "" then
		strNewGuild = String_GetWeaselString(Apollo.GetString("Nameplates_GuildDisplay"), strNewGuild)
	end
	
	--if strNewGuild ~= wndGuild:GetText() then
	--	wndGuild:SetTextRaw(strNewGuild)
	--end

	local bShowGuild =  self:GetSetting(tNameplate,"razShowGuild")
	bShowGuild = bShowGuild and strNewGuild ~= nil and strNewGuild ~= ""

	if bShowName and bShowGuild then
		wndName:SetText(strNewName)
		wndGuild:SetTextRaw(strNewGuild)
	else
		if bShowName and not bShowGuild then
			wndName:SetText("")
			wndGuild:SetText(strNewName)
		elseif not bShowName and bShowGuild then
			wndName:SetText("")
			wndGuild:SetTextRaw(strNewGuild)
		else
			wndName:SetText("")
			wndGuild:SetTextRaw("")
		end
		
	end
end

function RazorPlates:ShowLevel(tNameplate)
	local unitOwner = tNameplate.unitOwner
	local bShow =  self:GetSetting(tNameplate,"razShowLevel")
	tNameplate.wnd.level:Show(bShow)
	if bShow then
		tNameplate.wnd.level:SetText(unitOwner:GetLevel() or "")
		self:ColorLevel(tNameplate)
	end
end

function RazorPlates:ColorLevel(tNameplate)
	local unitPlayer = self.unitPlayer
	local unitOwner = tNameplate.unitOwner
	local wndNameplate = tNameplate.wndNameplate

	local eDisposition = tNameplate.eDisposition
	local nCon = self:HelperCalculateConValue(unitOwner)
	
	local crLevelColorToUse = karConColors[nCon][2]
	if unitOwner:GetLevel() == nil then
		crLevelColorToUse = karConColors[1][2]
	end
	
	local crColorToUse = karDisposition.tTextColors[eDisposition]
	local unitController = unitOwner:GetUnitOwner() or unitOwner
	local strUnitType = unitOwner:GetType()
	
	if strUnitType == "Player" or strUnitType == "Pet" or strUnitType == "Esper Pet" then
		if eDisposition == Unit.CodeEnumDisposition.Friendly or unitOwner:IsThePlayer() then
			crColorToUse = kcrUnflaggedAlly
			if unitController:IsPvpFlagged() then
				crColorToUse = kcrFlaggedAlly
			elseif unitController:IsInYourGroup() then
				crColorToUse = kcrUnflaggedGroupmate
			elseif tNameplate.bIsGuildMember then
				crColorToUse = kcrUnflaggedGuildmate
			end
		else
			local bIsUnitFlagged = unitController:IsPvpFlagged()
			local bAmIFlagged = GameLib.IsPvpFlagged()
			
			if not bAmIFlagged and not bIsUnitFlagged then
				crColorToUse = kcrUnflaggedEnemyWhenUnflagged
			elseif bAmIFlagged and not bIsUnitFlagged then
				crColorToUse = kcrUnflaggedEnemyWhenFlagged
			elseif not bAmIFlagged and bIsUnitFlagged then
				crColorToUse = kcrFlaggedEnemyWhenUnflagged
			elseif bAmIFlagged and bIsUnitFlagged then
				crColorToUse = kcrFlaggedEnemyWhenFlagged
			end
		end
	end

	if unitOwner:GetType() ~= "Player" and unitOwner:IsTagged() and not unitOwner:IsTaggedByMe() and not unitOwner:IsSoftKill() then
		crColorToUse = kcrDefaultTaggedColor
	end

	if unitOwner:IsDead() then
		crColorToUse = kcrDeadColor
		crLevelColorToUse = kcrDeadColor
	end
	
	tNameplate.wnd.level:SetTextColor(crLevelColorToUse)
end

function RazorPlates:ShowClassOrDifficulty(tNameplate)
	local unitOwner = tNameplate.unitOwner
	local classDiffWnd = tNameplate.wnd.classDifficulty

	--if not tNameplate.wnd.hpBar:IsShown() then return end
	
	if tNameplate.bIsAPlayer then
		local bNewShow =  razPerUnitTypeSettings["GeneralSettings"].razShowClassIcons.settingsVal and tNameplate.wnd.hpBar:IsShown()
		classDiffWnd:SetSprite(ktIdToClassSprite[unitOwner:GetClassId()])
		local strColor = strFormat("ff%s", razPerUnitTypeSettings["ColorSettings"][tNameplate.classId].razClassColor.settingsVal)
		classDiffWnd:SetBGColor(strColor)

		if bNewShow ~= classDiffWnd:IsShown() then
			classDiffWnd:Show(bNewShow)
		end
	else
		classDiffWnd:SetSprite(ktDifficultySprites[unitOwner:GetDifficulty()])
		classDiffWnd:SetBGColor("ffffffff")
		local bNewShow =  razPerUnitTypeSettings["GeneralSettings"].razShowDifficultyIcons.settingsVal and tNameplate.wnd.hpBar:IsShown()
		if bNewShow ~= classDiffWnd:IsShown() then
			classDiffWnd:Show(bNewShow)
		end
	end	
end

function RazorPlates:ShowInterruptArmor(tNameplate)

	local iArmor = tNameplate.wnd.interruptArmor
	
	local bShowInterruptArmor = razPerUnitTypeSettings["GeneralSettings"][tNameplate.eDisposition].razShowInterruptArmor.settingsVal

	if not bShowInterruptArmor or not tNameplate.wnd.hpBar:IsShown() then
		iArmor:Show(false) 
		return
	end
	
	local unitOwner = tNameplate.unitOwner
	local nArmor = tNameplate.unitOwner:GetInterruptArmorValue() or 0
	local nArmorMax = tNameplate.unitOwner:GetInterruptArmorMax()
	
	if iArmor then
		if nArmorMax == -1 then
			iArmor:SetText("X")
			iArmor:SetTextColor("cyan")
			iArmor:Show(true)
		elseif nArmor > 0 then
			iArmor:SetText(nArmor)
			iArmor:SetTextColor("white")
			iArmor:Show(true)
		else
			iArmor:Show(false)
		end
	end
end

function RazorPlates:ShowResourceBar(tNameplate)
	local unitOwner = tNameplate.unitOwner
	local bShow =  self:GetSetting(tNameplate,"razShowResourceBar")

	if bShow and tNameplate.bIsAPlayer then
		tNameplate.wnd.manaBar:Show(true)
		tNameplate.wnd.manaBar:SetBGColor(ktIdToResourceColor[tNameplate.classId])

		if (tNameplate.nLastResource == 0 and razPerUnitTypeSettings["StyleSettings"].razHideEmptyResourceBar.settingsVal) then
			tNameplate.wnd.manaBarBack:Show( false )
			tNameplate.wnd.manaBarHighlights:Show(false)
		else
			tNameplate.wnd.manaBarBack:Show( true )
			tNameplate.wnd.manaBarHighlights:Show(not razPerUnitTypeSettings["StyleSettings"].razMinimalisticMode.settingsVal)
		end
	else
		tNameplate.wnd.manaBar:Show(false)
		tNameplate.wnd.manaBarBack:Show(false)
		tNameplate.wnd.manaBarHighlights:Show(false)
	end
end

function RazorPlates:UpdateScaleAndTransparency(tNameplate)
	local scale =  self:GetSetting(tNameplate,"razScale")
	local transparency =  self:GetSetting(tNameplate,"razTransparency")
	local offset =  self:GetSetting(tNameplate,"razOffset")
	--Print(tNameplate.unitOwner:GetName())
	if tNameplate.wnd.hpBar:IsShown() then
		scale  = 0.01 * scale * (1.0 + tNameplate.bonusScale + tNameplate.LowHPBonusScale + tNameplate.CleanseBonusScale)
	else
		scale  = 0.01 * scale
	end

	transparency = 0.01 * transparency * (1.0 + tNameplate.bonusTransparency)
	
	tNameplate.wndNameplate:SetOpacity(transparency)
		
	tNameplate.wnd.nameplateParent:SetScale(scale)
	tNameplate.wnd.nameplateParent:SetAnchorOffsets(-150*scale , 0 + offset , -150 * scale + 300 , 126+ offset)
end


function RazorPlates:DrawThreat(tNameplate)
	if not razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatAlertEnabled.settingsVal or tNameplate.eDisposition > 1  then
		--tNameplate.bShowAggroMarker = false
		return
	end
	if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatAlertInInstanceOnly.settingsVal then
		if not GroupLib.InInstance() then
			return
		end
	end
	local aggroMarker = tNameplate.wnd.aggroMarker

	
	
	if tNameplate.unitOwner:IsDead() or tNameplate.unitOwner:GetTarget() == nil then
		--aggroMarker:Show(false)
		tNameplate.bShowAggroMarker = false
		tNameplate.bonusScale = 0
		tNameplate.bonusTransparency = 0
		tNameplate.threatHPColorChange = false
		return
	end
		
	if tNameplate.unitOwner:GetTarget() == self.unitPlayer then --unit targets player
		tNameplate.bShowAggroMarker = (razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razShowThreatMarker.settingsVal and not razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razTankModeEnabled.settingsVal)

		if tNameplate.bShowAggroMarker then
			aggroMarker:SetBGColor(strFormat("ff%s",razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatMarkerColor.settingsVal))
		end

		if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razTankModeEnabled.settingsVal then
			if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatChangeScale.settingsVal then
				tNameplate.bonusScale = 0
			end
			
			if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatChangeTransparency.settingsVal then
				tNameplate.bonusTransparency = - razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatTransparency.settingsVal * 0.01
			else
				tNameplate.bonusTransparency = 0
			end

			if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatChangeHPColor.settingsVal then
				tNameplate.threatHPColorChange = false
			end
		else
			if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatChangeScale.settingsVal then
				tNameplate.bonusScale = razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatScale.settingsVal * 0.01
			else
				tNameplate.bonusScale = 0
			end

			if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatChangeTransparency.settingsVal then
				tNameplate.bonusTransparency = 0
			end

			if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatChangeHPColor.settingsVal then
				tNameplate.threatHPColorChange = true
			else
				tNameplate.threatHPColorChange = false
			end
		end

	else --unit targets something else
		tNameplate.bShowAggroMarker =  (razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razShowThreatMarker.settingsVal and razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razTankModeEnabled.settingsVal)

		if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razTankModeEnabled.settingsVal then
			if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatChangeScale.settingsVal then
				tNameplate.bonusScale = razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatScale.settingsVal * 0.01
			end
			
			if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatChangeTransparency.settingsVal then
				tNameplate.bonusTransparency = 0
			else
				tNameplate.bonusTransparency = - razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatTransparency.settingsVal * 0.01
			end

			if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatChangeHPColor.settingsVal then
				tNameplate.threatHPColorChange = true
			end
		else
			if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatChangeScale.settingsVal then
				tNameplate.bonusScale = 0
			else
				tNameplate.bonusScale = razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatScale.settingsVal * 0.01
			end

			if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatChangeTransparency.settingsVal then
				tNameplate.bonusTransparency = - razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatTransparency.settingsVal * 0.01
			end

			if razPerUnitTypeSettings["AlertSettings"]["ThreatAlert"].razThreatChangeHPColor.settingsVal then
				tNameplate.threatHPColorChange = false
			else
				tNameplate.threatHPColorChange = true
			end
		end

	end
end


function RazorPlates:DrawLowHealth(tNameplate)
	aggroMarker = tNameplate.wnd.aggroMarker
	eDisposition = tNameplate.eDisposition
	if eDisposition == 1 then
		eDisposition = 0
	end
	if not razPerUnitTypeSettings["AlertSettings"]["LowHPAlert"][eDisposition].razLowHPAlertEnabled.settingsVal then
		tNameplate.bShowLowHPMarker = false
		return
	end

	local newBonusScale = 0
		
	if tNameplate.unitOwner:IsDead() then
		tNameplate.bShowLowHPMarker = false
		tNameplate.LowHPBonusScale = 0
		return
	else
		local nHealthCurr 	= tNameplate.unitOwner:GetHealth()
		local nHealthMax 	= tNameplate.unitOwner:GetMaxHealth()
		
		if nHealthCurr ~= nil and (nHealthCurr / nHealthMax <= razPerUnitTypeSettings["AlertSettings"]["LowHPAlert"][eDisposition].razLowHPThreshold.settingsVal * 0.01) and nHealthCurr > 0 then
			tNameplate.bShowLowHPMarker = (razPerUnitTypeSettings["AlertSettings"]["LowHPAlert"][eDisposition].razLowHPShowMarker.settingsVal)
			
			if tNameplate.bShowLowHPMarker then
				aggroMarker:SetBGColor(strFormat("ff%s",razPerUnitTypeSettings["AlertSettings"]["LowHPAlert"][eDisposition].razLowHPMarkerColor.settingsVal))
			end
			if razPerUnitTypeSettings["AlertSettings"]["LowHPAlert"][eDisposition].razLowHPChangeHPColor.settingsVal then
				tNameplate.LowHPColorChange = true
			else
				tNameplate.LowHPColorChange = false
			end
			if razPerUnitTypeSettings["AlertSettings"]["LowHPAlert"][eDisposition].razLowHPChangeScale.settingsVal then
				newBonusScale = razPerUnitTypeSettings["AlertSettings"]["LowHPAlert"][eDisposition].razLowHPScale.settingsVal * 0.01
			else
				newBonusScale = 0
			end
		else
			tNameplate.bShowLowHPMarker = false
			tNameplate.LowHPColorChange = false
			newBonusScale = 0
		end
	end	
	if newBonusScale ~= tNameplate.LowHPBonusScale then
		tNameplate.LowHPBonusScale = newBonusScale
		self:UpdateScaleAndTransparency(tNameplate)
	end
end

function RazorPlates:DrawCleanse(tNameplate)
	if not tNameplate.bIsThePlayer then
		if not tNameplate.bIsGroupMember then
			tNameplate.bShowCleanseMarker = false
			return
		end
	end
	
	if not razPerUnitTypeSettings["AlertSettings"]["CleanseAlert"].razCleanseAlertEnabled.settingsVal then
		tNameplate.bShowCleanseMarker = false
		return
	end

--	Print("IS A GROUP MEMBER!")

	local aggroMarker = tNameplate.wnd.aggroMarker

	local bCleansable = false
	
	

	local newBonusScale = 0

	if tNameplate.unitOwner:IsDead() then
		tNameplate.bShowCleanseMarker = false
		tNameplate.CleanseBonusScale = 0
		tNameplate.CleanseHPColorChange = false
		return
	else
		local unitBuffs = tNameplate.unitOwner:GetBuffs() 
		local unitDebuffs = unitBuffs["arHarmful"]
		
		if unitDebuffs then
			for _, buff in pairs(unitDebuffs) do
				if buff["splEffect"]:GetClass() == 38 then
					bCleansable = true
					--Print("Got something cleansable!")
					break
				end
			end
		end

		if bCleansable then
			tNameplate.bShowCleanseMarker = (razPerUnitTypeSettings["AlertSettings"]["CleanseAlert"].razShowCleanseMarker.settingsVal)
			if tNameplate.bShowCleanseMarker then
				aggroMarker:SetBGColor(strFormat("ff%s",razPerUnitTypeSettings["AlertSettings"]["CleanseAlert"].razCleanseMarkerColor.settingsVal))
			end
			if razPerUnitTypeSettings["AlertSettings"]["CleanseAlert"].razCleanseChangeHPColor.settingsVal then
				tNameplate.CleanseHPColorChange = true
			else
				tNameplate.CleanseHPColorChange = false
			end
			if razPerUnitTypeSettings["AlertSettings"]["CleanseAlert"].razCleanseChangeScale.settingsVal then
				newBonusScale = razPerUnitTypeSettings["AlertSettings"]["CleanseAlert"].razCleanseScale.settingsVal * 0.01
			else
				newBonusScale = 0
			end
		else
			tNameplate.bShowCleanseMarker = false
			tNameplate.CleanseHPColorChange = false
			newBonusScale = 0
		end
	end

	if newBonusScale ~= tNameplate.CleanseBonusScale then
		tNameplate.CleanseBonusScale = newBonusScale
		self:UpdateScaleAndTransparency(tNameplate)
	end
end



function RazorPlates:DrawTargeting(tNameplate)
	--local wndNameplate = tNameplate.wndNameplate
	--local unitOwner = tNameplate.unitOwner

	local bUseTarget = tNameplate.bIsTarget and razPerUnitTypeSettings["GeneralSettings"].razShowTargetMarker.settingsVal
	
	tNameplate.wnd.targetMarker:SetBGColor(karDisposition.tTextColors[tNameplate.eDisposition])

	if tNameplate.nVulnerableTime > 0 then
		tNameplate.wnd.targetMarker:SetBGColor("magenta")
	end

	if bUseTarget ~= tNameplate.wnd.targetMarker:IsShown() then
		tNameplate.bIsDirty = true
		tNameplate.bWasTarget = bUseTarget
		tNameplate.wnd.targetMarker:Show(bUseTarget)
	--	self:DrawNameplate(tNameplate)
	--self.curOnFrame(self)
	end
end

function RazorPlates:UpdateStyleGlobal()
	local orgWidth = self.nOrgTopBarRight - self.nOrgTopBarLeft
	local halfWidth = (razPerUnitTypeSettings["StyleSettings"].razNameplateWidth.settingsVal - orgWidth ) * 0.5
	--Print("Half:" .. halfWidth)
	--self.nHPFrameLeft = self.nOrgTopBarLeft - halfWidth
	--self.nHPFrameRight = self.nOrgTopBarRight --+ halfWidth
	self.nNameplateLeft = self.nOrgTopBarLeft - halfWidth

	
	self.nNameplateWidth = self.nOrgTopBarRight + halfWidth - self.nNameplateLeft

	self.nHPFrameHeight = razPerUnitTypeSettings["StyleSettings"].razTopBarHeight.settingsVal
	self.nShieldFrameHeight = razPerUnitTypeSettings["StyleSettings"].razMiddleBarHeight.settingsVal
	self.nResourceFrameHeight = razPerUnitTypeSettings["StyleSettings"].razBottomBarHeight.settingsVal
	self.nCastFrameHeight = razPerUnitTypeSettings["StyleSettings"].razCastBarHeight.settingsVal

	if razPerUnitTypeSettings["StyleSettings"].razShieldBarOnTop.settingsVal then
		self.nShieldFrameTop = self.nOrgTopBarTop
		self.nShieldFrameBottom = self.nShieldFrameTop + self.nShieldFrameHeight

		self.nHPFrameTop = self.nShieldFrameBottom - 5
		self.nHPFrameBottom = self.nHPFrameTop -2 + self.nHPFrameHeight

		self.nResourceFrameTop = self.nHPFrameBottom -5
		self.nResourceFrameBottom = self.nResourceFrameTop -2 + self.nResourceFrameHeight

		--if tNameplate.bIsAPlayer and self:GetSetting(tNameplate,"razShowResourceBar") then
		self.nCastFrameTop = self.nResourceFrameBottom -5
		self.nCastFrameBottom = self.nCastFrameTop -2 + self.nCastFrameHeight
		--else
		--	self.nCastTop = self.nHPFrameBottom -5
		--	self.nCastBottom = self.nCastTop -2 + razPerUnitTypeSettings["StyleSettings"].razCastBarHeight.settingsVal
		--end

	else
		self.nHPFrameTop = self.nOrgTopBarTop
		self.nHPFrameBottom = self.nHPFrameTop + self.nHPFrameHeight

		self.nShieldFrameTop = self.nHPFrameBottom - 5
		self.nShieldFrameBottom = self.nShieldFrameTop - 2 + self.nShieldFrameHeight

		self.nResourceFrameTop = self.nShieldFrameBottom -5
		self.nResourceFrameBottom = self.nResourceFrameTop -2 + self.nResourceFrameHeight

	--	if tNameplate.bIsAPlayer and self:GetSetting(tNameplate,"razShowResourceBar") then
		self.nCastFrameTop = self.nResourceFrameBottom -5
		self.nCastFrameBottom = self.nCastFrameTop -2 + self.nCastFrameHeight
	end

	if self.bPerspectivePlatesDetected and  self.addonPerspective.settings.perspectiveEnabled then
		if not razPerUnitTypeSettings["GeneralSettings"].razDoubleCastBarUpdateRate.settingsVal then
			self.curOnFrame = self.PerspectiveOnFrame
		--	Print("PerspectiveOnFrame")
		else
			self.curOnFrame = self.SmootherCastBarPerspectiveOnFrame
		--	Print("SmootherCastBarPerspectiveOnFrame")
		end
	else
		if not razPerUnitTypeSettings["GeneralSettings"].razDoubleCastBarUpdateRate.settingsVal then
			self.curOnFrame = self.NormalOnFrame
		--	Print("Normal Cast Bar Update Rate enabled")
		else
			self.curOnFrame = self.SmootherCastBarOnFrame
		--	Print("SmootherCastBarOnFrame")
		end
	end
end

function RazorPlates:UpdateStyle(tNameplate)
	--Print(tNameplate.unitOwner.GetName())
	--if tNameplate.unitOwner ~=nil then
	--	Print(tNameplate.unitOwner:GetName())
	--end

	local bGotShield = false
	local shieldMax = tNameplate.unitOwner:GetShieldCapacityMax()

	if shieldMax ~= nil then
		bGotShield = shieldMax > 0
	end
	if razPerUnitTypeSettings["StyleSettings"].razRectGFX.settingsVal then
		tNameplate.wnd.hpBar:SetSprite("barColorSquared")
		tNameplate.wnd.shieldBar:SetSprite("barColorSquared")
		tNameplate.wnd.absorbBar:SetSprite("barColorSquared")
		tNameplate.wnd.manaBar:SetSprite("barColorSquared")
		tNameplate.wnd.castBarFill:SetFullSprite("barColorSquared")

		tNameplate.wnd.hpBarBack:SetSprite("borderSquared")
		tNameplate.wnd.shieldBarBack:SetSprite("borderSquared")
		tNameplate.wnd.manaBarBack:SetSprite("borderSquared")
		tNameplate.wnd.castBarBack:SetSprite("borderSquared")

		tNameplate.wnd.hpBarHighlights:SetSprite("squaredHighlights")
		tNameplate.wnd.shieldBarHighlights:SetSprite("squaredHighlights")
		tNameplate.wnd.manaBarHighlights:SetSprite("squaredHighlights")
		tNameplate.wnd.castBarHighlights:SetSprite("squaredHighlights")
	else
		tNameplate.wnd.hpBar:SetSprite("barColor")
		tNameplate.wnd.shieldBar:SetSprite("barColor")
		tNameplate.wnd.absorbBar:SetSprite("barColor")
		tNameplate.wnd.manaBar:SetSprite("barColor")
		tNameplate.wnd.castBarFill:SetFullSprite("barColor")

		tNameplate.wnd.hpBarBack:SetSprite("borderRounded")
		tNameplate.wnd.shieldBarBack:SetSprite("borderRounded")
		tNameplate.wnd.manaBarBack:SetSprite("borderRounded")
		tNameplate.wnd.castBarBack:SetSprite("borderRounded")

		tNameplate.wnd.hpBarHighlights:SetSprite("defaultHighlights")
		tNameplate.wnd.shieldBarHighlights:SetSprite("defaultHighlights")
		tNameplate.wnd.manaBarHighlights:SetSprite("defaultHighlights")
		tNameplate.wnd.castBarHighlights:SetSprite("defaultHighlights")
	end

	tNameplate.wnd.shieldBar:SetBGColor(strFormat("ff%s", razPerUnitTypeSettings["ColorSettings"].razShieldBarColor.settingsVal))
	tNameplate.wnd.absorbBar:SetBGColor(strFormat("ff%s", razPerUnitTypeSettings["ColorSettings"].razAbsorbBarColor.settingsVal))

	tNameplate.wnd.hpBarHighlights:Show( not razPerUnitTypeSettings["StyleSettings"].razMinimalisticMode.settingsVal and self:GetSetting(tNameplate,"razShowBars")  )
	tNameplate.wnd.name:SetFont(razFonts[razPerUnitTypeSettings["StyleSettings"].razNameFont.settingsVal+1])
	local fntSize = razFontSizes[razPerUnitTypeSettings["StyleSettings"].razNameFont.settingsVal+1]
	local l,t,r,b = tNameplate.wnd.guild:GetAnchorOffsets()
	tNameplate.wnd.guild:SetAnchorOffsets(l,b - fntSize -4,r,b)
	--Print(razFonts[razPerUnitTypeSettings["StyleSettings"].razNameFont.settingsVal+1] .. ":" .. razFontSizes[razPerUnitTypeSettings["StyleSettings"].razNameFont.settingsVal+1])
	tNameplate.wnd.guild:SetFont(razFonts[razPerUnitTypeSettings["StyleSettings"].razNameFont.settingsVal+1])
	tNameplate.wnd.hpText1:SetFont(razFonts[razPerUnitTypeSettings["StyleSettings"].razHPTextFont.settingsVal+1])
	tNameplate.wnd.hpText2:SetFont(razFonts[razPerUnitTypeSettings["StyleSettings"].razHPTextFont.settingsVal+1])
	tNameplate.wnd.interruptArmor:SetFont(razFonts[razPerUnitTypeSettings["StyleSettings"].razLevelFont.settingsVal+1])
	tNameplate.wnd.level:SetFont(razFonts[razPerUnitTypeSettings["StyleSettings"].razLevelFont.settingsVal+1])
	tNameplate.wnd.hpText3:SetFont(razFonts[razPerUnitTypeSettings["StyleSettings"].razResourceBarFont.settingsVal+1])
	tNameplate.wnd.castBarText:SetFont(razFonts[razPerUnitTypeSettings["StyleSettings"].razCastBarFont.settingsVal+1])
	tNameplate.wnd.castBarNameText:SetFont(razFonts[razPerUnitTypeSettings["StyleSettings"].razCastBarFont.settingsVal+1])

	local hpBar = tNameplate.wnd.hpBarBack
	local shieldBar = tNameplate.wnd.shieldBarBack
	local resourceBar = tNameplate.wnd.manaBarBack
	local castBar = tNameplate.wnd.castBarBack
	local marker = tNameplate.wnd.aggroMarker

	if razPerUnitTypeSettings["GeneralSettings"].razUseShieldBar.settingsVal then
		if razPerUnitTypeSettings["StyleSettings"].razShieldBarOnTop.settingsVal and bGotShield then

			tNameplate.nShieldBarTop = self.nOrgTopBarTop
		
			tNameplate.nHPBarTop = tNameplate.nShieldBarTop + self.nShieldFrameHeight  - 5
			
			tNameplate.nResourceBarTop = tNameplate.nHPBarTop + self.nHPFrameHeight - 5
			
			if tNameplate.bIsAPlayer and self:GetSetting(tNameplate,"razShowResourceBar") then
				tNameplate.nCastBarTop = tNameplate.nResourceBarTop + self.nResourceFrameHeight - 5
			else
				tNameplate.nCastBarTop = tNameplate.nHPBarTop + self.nHPFrameHeight - 5
			end

		else
			tNameplate.nHPBarTop = self.nOrgTopBarTop
		
			tNameplate.nShieldBarTop = tNameplate.nHPBarTop + self.nHPFrameHeight  - 5

			if bGotShield then
				tNameplate.nResourceBarTop = tNameplate.nShieldBarTop + self.nShieldFrameHeight - 5

				if tNameplate.bIsAPlayer and self:GetSetting(tNameplate,"razShowResourceBar") then
					tNameplate.nCastBarTop = tNameplate.nResourceBarTop + self.nResourceFrameHeight - 5
				else
					tNameplate.nCastBarTop = tNameplate.nShieldBarTop + self.nShieldFrameHeight - 5
				end
			else
				tNameplate.nResourceBarTop = tNameplate.nHPBarTop + self.nHPFrameHeight - 5

				if tNameplate.bIsAPlayer and self:GetSetting(tNameplate,"razShowResourceBar") then
					tNameplate.nCastBarTop = tNameplate.nResourceBarTop + self.nResourceFrameHeight - 5
				else
					tNameplate.nCastBarTop = tNameplate.nHPBarTop + self.nHPFrameHeight - 5
				end
			end

		end
	else
		tNameplate.nHPBarTop = self.nOrgTopBarTop
		
		tNameplate.nShieldBarTop = tNameplate.nHPBarTop + self.nHPFrameHeight  - 5
		
		tNameplate.nResourceBarTop = tNameplate.nHPBarTop + self.nHPFrameHeight - 5

		if tNameplate.bIsAPlayer and self:GetSetting(tNameplate,"razShowResourceBar") then
			tNameplate.nCastBarTop = tNameplate.nResourceBarTop + self.nResourceFrameHeight - 5
		else
		--	if tNameplate.bGotShield then
		--		tNameplate.nCastBarTop = tNameplate.nShieldBarTop + self.nShieldFrameHeight - 5
		--	else
				tNameplate.nCastBarTop =  tNameplate.nHPBarTop + self.nHPFrameHeight  - 5
		--	end
		end
	end
--[[
	if razPerUnitTypeSettings["GeneralSettings"].razUseShieldBar.settingsVal and razPerUnitTypeSettings["StyleSettings"].razShieldBarOnTop.settingsVal and bGotShield then
		
		tNameplate.nShieldBarTop = self.nOrgTopBarTop
		
		tNameplate.nHPBarTop = tNameplate.nShieldBarTop + self.nShieldFrameHeight  - 5
		
		tNameplate.nResourceBarTop = tNameplate.nHPBarTop + self.nHPFrameHeight - 5
		
		if tNameplate.bIsAPlayer and self:GetSetting(tNameplate,"razShowResourceBar") then
			tNameplate.nCastBarTop = tNameplate.nResourceBarTop + self.nResourceFrameHeight - 5
		else
			tNameplate.nCastBarTop = tNameplate.nHPBarTop + self.nHPFrameHeight - 5
		end
		--else
		--	self.nCastTop = self.nHPFrameBottom -5
		--	self.nCastBottom = self.nCastTop -2 + razPerUnitTypeSettings["StyleSettings"].razCastBarHeight.settingsVal
		--end
		--
	else
		tNameplate.nHPBarTop = self.nOrgTopBarTop
		
		tNameplate.nShieldBarTop = tNameplate.nHPBarTop + self.nHPFrameHeight  - 5
		
		if not razPerUnitTypeSettings["GeneralSettings"].razUseShieldBar.settingsVal then
			tNameplate.nResourceBarTop = tNameplate.nHPBarTop + self.nHPFrameHeight - 5

			if tNameplate.bIsAPlayer and self:GetSetting(tNameplate,"razShowResourceBar") then
				tNameplate.nCastBarTop = tNameplate.nResourceBarTop + self.nResourceFrameHeight - 5
			else
				if tNameplate.bGotShield then
					tNameplate.nCastBarTop = tNameplate.nShieldBarTop + self.nShieldFrameHeight - 5
				else
					tNameplate.nCastBarTop =  tNameplate.nHPBarTop + self.nHPFrameHeight  - 5
				end
			end
		else
			tNameplate.nResourceBarTop = tNameplate.nShieldBarTop + self.nShieldFrameHeight - 5

			if tNameplate.bIsAPlayer and self:GetSetting(tNameplate,"razShowResourceBar") then
				tNameplate.nCastBarTop = tNameplate.nResourceBarTop + self.nResourceFrameHeight - 5
			else
				if tNameplate.bGotShield then
					tNameplate.nCastBarTop = tNameplate.nShieldBarTop + self.nShieldFrameHeight - 5
				else
					tNameplate.nCastBarTop =  tNameplate.nHPBarTop + self.nHPFrameHeight  - 5
				end
			end
		end
	end
]]--
	--tNameplate.nHPBarTop = self.nHPFrameTop
--	tNameplate.nShieldBarTop = self.nShieldFrameTop
--	tNameplate.nResourceBarTop = self.nResourceFrameTop
--	tNameplate.nCastBarTop = self.nCastFrameTop

	hpBar:SetAnchorOffsets(self.nNameplateLeft, tNameplate.nHPBarTop, self.nNameplateLeft + self.nNameplateWidth-1, tNameplate.nHPBarTop + self.nHPFrameHeight)
	shieldBar:SetAnchorOffsets(0, tNameplate.nShieldBarTop, 0, tNameplate.nShieldBarTop + self.nShieldFrameHeight )
	resourceBar:SetAnchorOffsets(0, tNameplate.nResourceBarTop, 0,tNameplate.nResourceBarTop + self.nResourceFrameHeight  )
	castBar:SetAnchorOffsets(0, tNameplate.nCastBarTop, 0, tNameplate.nCastBarTop + self.nCastFrameHeight)

	if razPerUnitTypeSettings["GeneralSettings"].razUseShieldBar.settingsVal and bGotShield then
		if razPerUnitTypeSettings["StyleSettings"].razShieldBarOnTop.settingsVal then
			marker:SetAnchorOffsets(-10,self.nOrgTopBarTop-10, 10,  tNameplate.nHPBarTop + self.nHPFrameHeight +10)
		else
			marker:SetAnchorOffsets(-10,self.nOrgTopBarTop-10, 10, tNameplate.nShieldBarTop + self.nShieldFrameHeight +10)
		end
	else
		marker:SetAnchorOffsets(-10,self.nOrgTopBarTop-10, 10, tNameplate.nHPBarTop + self.nHPFrameHeight +10)
	end
	

	local bHPTextInMiddle = razPerUnitTypeSettings["StyleSettings"].razHPTextCentered.settingsVal --and razPerUnitTypeSettings["GeneralSettings"].razUseShieldBar.settingsVal

	if bHPTextInMiddle then
		tNameplate.wnd.hpText1:SetAnchorOffsets(self.nNameplateLeft, tNameplate.nHPBarTop, self.nNameplateLeft + self.nNameplateWidth,tNameplate.nHPBarTop + self.nHPFrameHeight)
		tNameplate.wnd.hpText2:SetAnchorOffsets(self.nNameplateLeft,tNameplate.nShieldBarTop,self.nNameplateLeft + self.nNameplateWidth,tNameplate.nShieldBarTop + self.nShieldFrameHeight+1)
		tNameplate.wnd.hpText1:SetTextFlags("DT_CENTER",true)
		tNameplate.wnd.hpText2:SetTextFlags("DT_CENTER",true)
	else
		local left = self.nNameplateLeft + self.nNameplateWidth
		tNameplate.wnd.hpText1:SetAnchorOffsets(left, tNameplate.nHPBarTop, left + self.nNameplateWidth,tNameplate.nHPBarTop + self.nHPFrameHeight)
		tNameplate.wnd.hpText2:SetAnchorOffsets(left, tNameplate.nShieldBarTop, left + self.nNameplateWidth,tNameplate.nShieldBarTop + self.nShieldFrameHeight+1)
		tNameplate.wnd.hpText1:SetTextFlags("DT_CENTER",false)
		tNameplate.wnd.hpText2:SetTextFlags("DT_CENTER",false)
	end
	tNameplate.wnd.hpText3:SetAnchorOffsets(self.nNameplateLeft, tNameplate.nResourceBarTop, self.nNameplateLeft + self.nNameplateWidth,tNameplate.nResourceBarTop + self.nResourceFrameHeight)
end


function RazorPlates:GetClassResource(unit)
	local classID = unit:GetClassId()

	if classID == GameLib.CodeEnumClass.Engineer or	classID == GameLib.CodeEnumClass.Warrior then
		return unit:GetResource(1), unit:GetMaxResource(1)
	elseif classID == GameLib.CodeEnumClass.Stalker then
		return unit:GetResource(3), unit:GetMaxResource(3)
	else
		return unit:GetMana(), unit:GetMaxMana()
	end
end


function RazorPlates:FormatBigNumber(nArg)
	local strResult = ""
	if nArg < 1000 then
		return tostring(math.floor(nArg))
	elseif nArg < 1000000 then
		return strFormat("%.1fk", nArg*0.001)
	elseif nArg < 1000000000 then
		return strFormat("%.1fm", nArg*0.000001)
	elseif nArg < 1000000000000 then
		return strFormat("%.1fb", nArg*0.000000001)
	end
	return strResult
end

function RazorPlates:HelperCalculateConValue(unitTarget)
	if unitTarget == nil or self.unitPlayer == nil then
		return 1
	end

	local nUnitCon = self.unitPlayer:GetLevelDifferential(unitTarget)

	local nCon = 1 --default setting

	if nUnitCon <= karConColors[1][1] then -- lower bound
		nCon = 1
	elseif nUnitCon >= karConColors[#karConColors][1] then -- upper bound
		nCon = #karConColors
	else
		for idx = 2, (#karConColors - 1) do -- everything in between
			if nUnitCon == karConColors[idx][1] then
				nCon = idx
			end
		end
	end

	return nCon
end


function RazorPlates:DrawRewards(tNameplate)
	local wndNameplate = tNameplate.wndNameplate
	local unitOwner = tNameplate.unitOwner
	
	--local bUseTarget = tNameplate.bIsTarget
	local bShow = true --self.bShowRewardsMain
	--if bUseTarget then
	--	bShow = self.bShowRewardsTarget
	--end
	
	tNameplate.wnd.questRewards:Show(bShow)
	local tRewardsData = tNameplate.wnd.questRewards:GetData()
	if bShow and tRewardsData ~= nil and tRewardsData.nIcons ~= nil and tRewardsData.nIcons > 0 then

		local strName = ""
		local wndnameRewardContainer = tNameplate.wnd.nameRewardContainer
		
		local anchorToNameOrGuild = nil

		if tNameplate.wnd.name:GetText() ~= "" then
			anchorToNameOrGuild = tNameplate.wnd.name
			strName = tNameplate.wnd.name:GetText()
		else
			anchorToNameOrGuild = tNameplate.wnd.guild
			strName = tNameplate.wnd.guild:GetText()
		end
		local nLeft, nTop, nRight, nBottom = anchorToNameOrGuild:GetAnchorOffsets()

		local nNameWidth = Apollo.GetTextWidth(razFonts[razPerUnitTypeSettings["StyleSettings"].razNameFont.settingsVal+1], strName)
		local nHalfNameWidth = nNameWidth / 2
			
		wndnameRewardContainer:SetAnchorOffsets(nHalfNameWidth, nTop, nHalfNameWidth + wndnameRewardContainer:ArrangeChildrenHorz(0), nBottom)
	end
end


function RazorPlates:CheckDrawDistance(tNameplate)
	local unitPlayer = self.unitPlayer
	local unitOwner = tNameplate.unitOwner
	
	if not unitOwner or not unitPlayer then
	    return false
	end

	local tPosTarget = unitOwner:GetPosition()
	local tPosPlayer = unitPlayer:GetPosition()

	if tPosTarget == nil then
		return
	end

	local nDeltaX = tPosTarget.x - tPosPlayer.x
	local nDeltaY = tPosTarget.y - tPosPlayer.y
	local nDeltaZ = tPosTarget.z - tPosPlayer.z

	local nDistance = (nDeltaX * nDeltaX) + (nDeltaY * nDeltaY) + (nDeltaZ * nDeltaZ)

	if tNameplate.bIsTarget or tNameplate.bIsCluster then
		bInRange = nDistance < knTargetRange
		return bInRange
	else
		local drawDistance
		if razPerUnitTypeSettings["GeneralSettings"][tNameplate.eDisposition].razOverrideDrawDistance.settingsVal then
			drawDistance = razPerUnitTypeSettings["GeneralSettings"][tNameplate.eDisposition].razDrawDistance.settingsVal
		else
			drawDistance = razPerUnitTypeSettings["GeneralSettings"].razDrawDistance.settingsVal
		end
		bInRange = nDistance < (drawDistance * drawDistance) -- squaring for quick maths
		return bInRange
	end
end

function RazorPlates:EvaluateVisibility(tNameplate)
	local unitPlayer = self.unitPlayer
	local unitOwner = tNameplate.unitOwner
	local eDisposition = tNameplate.eDisposition

	if not tNameplate.bOnScreen or tNameplate.bGibbed or (razPerUnitTypeSettings["GeneralSettings"].razUseOcclusion.settingsVal and tNameplate.bOccluded) then
		return false
	end
	
	local bHiddenUnit = not unitOwner:ShouldShowNamePlate()

	if bHiddenUnit and not tNameplate.bIsTarget then
		return false
	end
	
	if unitOwner:IsDead() then
		if tNameplate.bIsAPlayer and tNameplate.eDisposition == 2 then
		else
			return false
		end
	end

	if tNameplate.eDisposition == 2 and razPerUnitTypeSettings["GeneralSettings"].razHideForFriendlySpeechbubble.settingsVal and tNameplate.bSpeechBubble then
		return false
	end
			
	return true 
end

-----------------------------------------------------------------------------------------------
-- Nameplate Events
-----------------------------------------------------------------------------------------------

function RazorPlates:OnNameplateNameClick(wndHandler, wndCtrl, eMouseButton)
	local tNameplate = self.arWnd2Nameplate[wndHandler:GetId()]
	if tNameplate == nil then
		return
	end
	
	local unitOwner = tNameplate.unitOwner
	if GameLib.GetTargetUnit() ~= unitOwner and eMouseButton == GameLib.CodeEnumInputMouse.Left then
		GameLib.SetTargetUnit(unitOwner)
	end
end

function RazorPlates:OnWorldLocationOnScreen(wndHandler, wndControl, bOnScreen)
	local tNameplate = self.arWnd2Nameplate[wndHandler:GetId()]
	if tNameplate ~= nil then
		tNameplate.bOnScreen = bOnScreen
		self:UpdateNameplateVisibility(tNameplate)
	end
end

function RazorPlates:OnUnitOcclusionChanged(wndHandler, wndControl, bOccluded)
	local tNameplate = self.arWnd2Nameplate[wndHandler:GetId()]
	if tNameplate ~= nil then
		tNameplate.bOccluded = bOccluded
		self:UpdateNameplateVisibility(tNameplate)
	end
end

-----------------------------------------------------------------------------------------------
-- System Events
-----------------------------------------------------------------------------------------------

function RazorPlates:OnUnitTextBubbleToggled(tUnitArg, strText, nRange)
	local tNameplate = self.arUnit2Nameplate[tUnitArg:GetId()]
	if tNameplate ~= nil then
		tNameplate.bSpeechBubble = strText ~= nil and strText ~= ""
	--	self:UpdateNameplateVisibility(tNameplate)
	end
end

function RazorPlates:OnEnteredCombat(unitChecked, bInCombat)
	local tNameplate = self.arUnit2Nameplate[unitChecked:GetId()]
	if tNameplate ~= nil then
		if tNameplate.bInCombat ~= bInCombat then
			tNameplate.bInCombat = bInCombat
			tNameplate.bIsDirty = true
		end
	end
	if unitChecked == self.unitPlayer then
		if self.bPlayerInCombat ~= bInCombat then
			self.bPlayerInCombat = bInCombat
			--if razPerUnitTypeSettings["GeneralSettings"].razInCombatIfPlayerInCombat.settingsVal then 
			--	self:UpdateAllPlates()
			--end
			if tNameplate ~= nil then
				tNameplate.bIsDirty = true
			end
		end
		return
	end
end

function RazorPlates:UpdateSettingsPointer(tNameplate)
	if razPerUnitTypeSettings[true][tNameplate.unitType] ~= nil then
		if razPerUnitTypeSettings[true][tNameplate.unitType][tNameplate.eDisposition] ~= nil then

			tNameplate.Settings = {
										[true] = razPerUnitTypeSettings[true][tNameplate.unitType][tNameplate.eDisposition],
										[false] = razPerUnitTypeSettings[false][tNameplate.unitType][tNameplate.eDisposition],
									}
		end
	end
end


function RazorPlates:OnUnitGibbed(unitUpdated)
	local tNameplate = self.arUnit2Nameplate[unitUpdated:GetId()]
	if tNameplate ~= nil then
		tNameplate.bGibbed = true
	--	self:UpdateNameplateVisibility(tNameplate)
	end
end

function RazorPlates:OnUnitNameChanged(unitUpdated, strNewName)
	local tNameplate = self.arUnit2Nameplate[unitUpdated:GetId()]
	if tNameplate ~= nil then
		self:ShowGuildAndName(tNameplate)
	end
end

function RazorPlates:OnUnitTitleChanged(unitUpdated)
	local tNameplate = self.arUnit2Nameplate[unitUpdated:GetId()]
	if tNameplate ~= nil then
		self:ShowGuildAndName(tNameplate)
	end
end

function RazorPlates:OnPlayerTitleChanged()
	local tNameplate = self.arUnit2Nameplate[self.unitPlayer:GetId()]
	if tNameplate ~= nil then
		self:ShowGuildAndName(tNameplate)
	end
end


function RazorPlates:OnUnitLevelChanged(unitUpdating)
	local tNameplate = self.arUnit2Nameplate[unitUpdating:GetId()]
	if tNameplate ~= nil then
		self:ShowLevel(tNameplate)
	end
end

function RazorPlates:OnGuildChange()
	self.guildDisplayed = nil
	self.guildWarParty = nil
	for key, guildCurr in pairs(GuildLib.GetGuilds()) do
		local eGuildType = guildCurr:GetType()
		if eGuildType == GuildLib.GuildType_Guild then
			self.guildDisplayed = guildCurr
		end
		if eGuildType == GuildLib.GuildType_WarParty then
			self.guildWarParty = guildCurr
		end
	end
	
	for key, tNameplate in pairs(self.arUnit2Nameplate) do
		local unitOwner = tNameplate.unitOwner
		tNameplate.bIsGuildMember = self.guildDisplayed and self.guildDisplayed:IsUnitMember(unitOwner) or false
		tNameplate.bIsWarPartyMember = self.guildWarParty and self.guildWarParty:IsUnitMember(unitOwner) or false
	end
end

function RazorPlates:OnUnitGuildNameplateChanged(unitUpdated)
	local tNameplate = self.arUnit2Nameplate[unitUpdated:GetId()]
	if tNameplate ~= nil then
		self:ShowGuildAndName(tNameplate)
		self:UpdateGroupStatus(tNameplate)
	end
end

function RazorPlates:OnUnitMemberOfGuildChange(unitOwner)
	local tNameplate = self.arUnit2Nameplate[unitOwner:GetId()]
	if tNameplate ~= nil then
		self:ShowGuildAndName(tNameplate)
		self:UpdateGroupStatus(tNameplate)
		tNameplate.bIsGuildMember = self.guildDisplayed and self.guildDisplayed:IsUnitMember(unitOwner) or false
		tNameplate.bIsWarPartyMember = self.guildWarParty and self.guildWarParty:IsUnitMember(unitOwner) or false 
	end
end

function RazorPlates:OnTargetUnitChanged(unitOwner) -- build targeted options here; we get this event when a creature attacks, too
	for idx, tNameplateOther in pairs(self.arUnit2Nameplate) do
		local bIsTarget = tNameplateOther.bIsTarget
		local bIsCluster = tNameplateOther.bIsCluster
		
		tNameplateOther.bIsTarget = false
		tNameplateOther.bIsCluster = false
		
		if bIsTarget or bIsCluster then
			--self:ShowName(tNameplateOther)
			--self:ShowGuildAndName(tNameplateOther)
			--self:ShowLevel(tNameplateOther)
			self:UpdateNameplateRewardInfo(tNameplateOther)
		end
	end
	
	if unitOwner == nil then
		return
	end
	local tNameplate = self.arUnit2Nameplate[unitOwner:GetId()]
	if tNameplate == nil then
		return
	end

	if GameLib.GetTargetUnit() == unitOwner then
		tNameplate.bIsTarget = true
		--Print("Target now:" .. unitOwner:GetName())
		--self:ShowName(tNameplate)
		--self:ShowGuildAndName(tNameplate)
	--	self:ShowLevel(tNameplate)
		self:UpdateNameplateRewardInfo(tNameplate)
		
		local tCluster = unitOwner:GetClusterUnits()
		if tCluster ~= nil then
			tNameplate.bIsCluster = true
			
			for idx, unitCluster in pairs(tCluster) do
				local tNameplateOther = self.arUnit2Nameplate[unitCluster:GetId()]
				if tNameplateOther ~= nil then
					tNameplateOther.bIsCluster = true
				end
			end
		end
	end

	--tNameplate.bIsDirty = true
	--self.bAllDirty = true
end

-----------------------------------------------------------------------------------------------
-- Options
-----------------------------------------------------------------------------------------------
function RazorPlates:OnConfigure()
	self:OnRazorPlatesOn()
	--self:DeepPrint(self.arUnit2Nameplate[self.unitPlayer:GetId()])

	--for property,tData in pairs(karSavedProperties) do
	--	if tSavedData[property] ~= nil then
	--		self[property] = tSavedData[property]
	--	end
	--end

	--for property , tData in pairs(razProfiles[1].Settings) do
	--	Print(tostring(property) .. ":" .. tostring(tData))
	--end
	local wndControl = self.wndMain:FindChild("Characters")
		if wndControl:GetSelectedText() == nil then
			for i, s in pairs(ProfilesSaved) do
				wndControl:AddItem(i)
			end
			wndControl:SelectItemByIndex(1)
	end
end

function RazorPlates:OnRazorPlatesOn()
	local ePath = PlayerPathLib.GetPlayerPathType()
--	self.wndOptionsMain:FindChild("ShowRewardTypeMission"):FindChild("Icon"):SetSprite(karPathSprite[ePath])
	if self.wndMain ~= nil then 
		self.wndMain:Show(true)
		self:RefreshNameplatesConfigure()
	end
end

function RazorPlates:RefreshNameplatesConfigure()
	self:RefreshRazSettings(razPerUnitTypeSettings)
	self:RefreshRequirements(razPerUnitTypeSettings)
end

-- when the OK button is clicked
function RazorPlates:OnOK()
	self.wndMain:Show(false) -- hide the window
end

-- when the Cancel button is clicked
function RazorPlates:OnCancel()
	self.wndMain:Show(false) -- hide the window
end

---------------------------------------------------------------------------------------------------
-- PerUnitTypeModule Functions
---------------------------------------------------------------------------------------------------

function RazorPlates:Convert_CColor_To_String(c)
	return strFormat("%02X%02X%02X", math.floor(c.r * 255 + 0.5), math.floor(c.g * 255 + 0.5), math.floor(c.b * 255 + 0.5))
end

function RazorPlates:Convert_String_To_CColor(hex)
	local r, g, b = 0, 0, 0 -- invalid strings will result in these values being returned
	local n = tonumber(hex, 16)
	if n then r = math.floor(n / 65536); g = math.floor(n / 256) % 256; b = n % 256 end
	return CColor.new(r / 255, g / 255, b / 255, 1)
end

function RazorPlates:OnColorButton( wndHandler, wndControl, strText )
	ChosenColor = self:Convert_String_To_CColor(wndControl:GetText())
	--Print(wndControl:GetName())
	--ChosenColor = CColor.new(1,1,1,1)
	ColorPicker.AdjustCColor(ChosenColor, false, self.OnColorButtonChangeColor, {self, wndControl})
	wndControl:SetBGColor(ChosenColor)
end

function RazorPlates.OnColorButtonChangeColor( args )
	args[2]:SetBGColor(ChosenColor)
	args[2]:SetText(args[1]:Convert_CColor_To_String(ChosenColor))
	args[1]:SetVal(razPerUnitTypeSettings, args[2]:GetData(),args[2]:GetText())
end

function RazorPlates:OnGenericSliderChange( wndHandler, wndControl, fNewValue, fOldValue )
--	Print(strFormat("Handler:%s Control:%s",wndHandler:GetName(),wndControl:GetName()))
	if wndHandler == wndControl then
		self:SetVal(razPerUnitTypeSettings, wndControl:GetData(), wndControl:GetValue())
		local info = wndControl:GetPixieInfo(1)
		info.strText = tostring(wndControl:GetValue())
		wndControl:UpdatePixie(1,info)
	end
end

function RazorPlates:OnGenericComboBoxChanged( wndHandler, wndControl )
--	Print(strFormat("Handler:%s Control:%s",wndHandler:GetName(),wndControl:GetName()))
	self:SetVal(razPerUnitTypeSettings, wndControl:GetData(), wndControl:GetSelectedIndex())
end

function RazorPlates:OnGenericButtonCheck(wndHandler, wndControl, eMouseButton)
--	Print(strFormat("Handler:%s Control:%s",wndHandler:GetName(),wndControl:GetName()))
	local strSettingName = wndControl:GetData()
	if strSettingName ~= nil then
		self:SetVal(razPerUnitTypeSettings, strSettingName, wndControl:IsChecked() )
		self:RefreshRequirements(razPerUnitTypeSettings)
	end
end




---------------------------------------------------------------------------------------------------
-- ProfilesModule Functions
---------------------------------------------------------------------------------------------------

function RazorPlates:OnLoadProfile1( wndHandler, wndControl, eMouseButton )
--	local wndControl = self.wndMain:FindChild("Characters")
--	local prof = ProfilesSaved[wndControl:GetSelectedText()]
--	self:LoadRazSettings(razPerUnitTypeSettings, prof)
--	self:RefreshRazSettings(razPerUnitTypeSettings)
--	self.bAllDirty = true
	--self:RazSettings(razPerUnitTypeSettings, prof1)
	local wndControl = self.wndMain:FindChild("Characters")
	if wndControl:GetSelectedText() == nil then
		return
	end
	self.wndConfirmation:Show(true)
	self.wndConfirmation:ToFront()
end

---------------------------------------------------------------------------------------------------
-- ConfirmationDialog Functions
---------------------------------------------------------------------------------------------------

function RazorPlates:OnLoadProfileYes( wndHandler, wndControl, eMouseButton )
	local wndControl = self.wndMain:FindChild("Characters")
	if wndControl:GetSelectedText() == nil then
		return
	end
	local prof = ProfilesSaved[wndControl:GetSelectedText()]
	self:LoadRazSettings(razPerUnitTypeSettings, prof)
	self:RefreshRazSettings(razPerUnitTypeSettings)
	self.bAllDirty = true
	self.wndConfirmation:Show(false)
end

function RazorPlates:OnLoadProfileNo( wndHandler, wndControl, eMouseButton )
	self.wndConfirmation:Show(false)
end

-----------------------------------------------------------------------------------------------
-- RazorPlates Instance
-----------------------------------------------------------------------------------------------
local RazorPlatesInst = RazorPlates:new()
RazorPlatesInst:Init()
